from __future__ import absolute_import, division

import unittest
import numpy as np
from astropy import constants
import lal

import bilby
from bilby.core import utils


class TestConstants(unittest.TestCase):

    def test_speed_of_light(self):
        self.assertEqual(utils.speed_of_light, lal.C_SI)
        self.assertLess(
           abs(utils.speed_of_light - constants.c.value) / utils.speed_of_light,
           1e-16)

    def test_parsec(self):
        self.assertEqual(utils.parsec, lal.PC_SI)
        self.assertLess(
           abs(utils.parsec - constants.pc.value) / utils.parsec,
           1e-11)

    def test_solar_mass(self):
        self.assertEqual(utils.solar_mass, lal.MSUN_SI)
        self.assertLess(
            abs(utils.solar_mass - constants.M_sun.value) / utils.solar_mass,
            1e-4)

    def test_radius_of_earth(self):
        self.assertEqual(bilby.core.utils.radius_of_earth, lal.REARTH_SI)
        self.assertLess(
            abs(utils.radius_of_earth - constants.R_earth.value) / utils.radius_of_earth,
            1e-5)


class TestFFT(unittest.TestCase):

    def setUp(self):
        self.sampling_frequency = 10

    def tearDown(self):
        del self.sampling_frequency

    def test_nfft_sine_function(self):
        injected_frequency = 2.7324
        duration = 100
        times = utils.create_time_series(self.sampling_frequency, duration)

        time_domain_strain = np.sin(2 * np.pi * times * injected_frequency + 0.4)

        frequency_domain_strain, frequencies = bilby.core.utils.nfft(time_domain_strain, self.sampling_frequency)
        frequency_at_peak = frequencies[np.argmax(np.abs(frequency_domain_strain))]
        self.assertAlmostEqual(injected_frequency, frequency_at_peak, places=1)

    def test_nfft_infft(self):
        time_domain_strain = np.random.normal(0, 1, 10)
        frequency_domain_strain, _ = bilby.core.utils.nfft(time_domain_strain, self.sampling_frequency)
        new_time_domain_strain = bilby.core.utils.infft(frequency_domain_strain, self.sampling_frequency)
        self.assertTrue(np.allclose(time_domain_strain, new_time_domain_strain))


class TestInferParameters(unittest.TestCase):

    def setUp(self):
        def source_function(freqs, a, b, *args, **kwargs):
            return None

        class TestClass:
            def test_method(self, a, b, *args, **kwargs):
                pass

        class TestClass2:
            def test_method(self, freqs, a, b, *args, **kwargs):
                pass

        self.source1 = source_function
        test_obj = TestClass()
        self.source2 = test_obj.test_method
        test_obj2 = TestClass2()
        self.source3 = test_obj2.test_method

    def tearDown(self):
        del self.source1
        del self.source2

        
    def test_args_kwargs_handling(self):
        expected = ['a', 'b']
        actual = utils.infer_parameters_from_function(self.source1)
        self.assertListEqual(expected, actual)

    def test_self_handling(self):
        expected = ['a', 'b']
        actual = utils.infer_args_from_method(self.source2)
        self.assertListEqual(expected, actual)
        
    def test_self_handling_method_as_function(self):
        expected = ['a', 'b']
        actual = utils.infer_parameters_from_function(self.source3)
        self.assertListEqual(expected, actual)

class TestTimeAndFrequencyArrays(unittest.TestCase):

    def setUp(self):
        self.start_time = 1.3
        self.sampling_frequency = 5
        self.duration = 1.6
        self.frequency_array = utils.create_frequency_series(sampling_frequency=self.sampling_frequency,
                                                             duration=self.duration)
        self.time_array = utils.create_time_series(sampling_frequency=self.sampling_frequency,
                                                   duration=self.duration,
                                                   starting_time=self.start_time)

    def tearDown(self):
        del self.start_time
        del self.sampling_frequency
        del self.duration
        del self.frequency_array
        del self.time_array

    def test_create_time_array(self):
        expected_time_array = np.array([1.3, 1.5, 1.7, 1.9, 2.1, 2.3, 2.5, 2.7])
        time_array = utils.create_time_series(sampling_frequency=self.sampling_frequency,
                                              duration=self.duration, starting_time=self.start_time)
        self.assertTrue(np.allclose(expected_time_array, time_array))

    def test_create_frequency_array(self):
        expected_frequency_array = np.array([0.0, 0.625, 1.25, 1.875, 2.5])
        frequency_array = utils.create_frequency_series(sampling_frequency=self.sampling_frequency,
                                                        duration=self.duration)
        self.assertTrue(np.allclose(expected_frequency_array, frequency_array))

    def test_get_sampling_frequency_from_time_array(self):
        new_sampling_freq, _ = utils.get_sampling_frequency_and_duration_from_time_array(
            self.time_array)
        self.assertEqual(self.sampling_frequency, new_sampling_freq)

    def test_get_sampling_frequency_from_time_array_unequally_sampled(self):
        self.time_array[-1] += 0.0001
        with self.assertRaises(ValueError):
            _, _ = utils.get_sampling_frequency_and_duration_from_time_array(self.time_array)

    def test_get_duration_from_time_array(self):
        _, new_duration = utils.get_sampling_frequency_and_duration_from_time_array(self.time_array)
        self.assertEqual(self.duration, new_duration)

    def test_get_start_time_from_time_array(self):
        new_start_time = self.time_array[0]
        self.assertEqual(self.start_time, new_start_time)

    def test_get_sampling_frequency_from_frequency_array(self):
        new_sampling_freq, _ = utils.get_sampling_frequency_and_duration_from_frequency_array(
            self.frequency_array)
        self.assertEqual(self.sampling_frequency, new_sampling_freq)

    def test_get_sampling_frequency_from_frequency_array_unequally_sampled(self):
        self.frequency_array[-1] += 0.0001
        with self.assertRaises(ValueError):
            _, _ = utils.get_sampling_frequency_and_duration_from_frequency_array(self.frequency_array)

    def test_get_duration_from_frequency_array(self):
        _, new_duration = utils.get_sampling_frequency_and_duration_from_frequency_array(
            self.frequency_array)
        self.assertEqual(self.duration, new_duration)

    def test_consistency_time_array_to_time_array(self):
        new_sampling_frequency, new_duration = \
            utils.get_sampling_frequency_and_duration_from_time_array(self.time_array)
        new_start_time = self.time_array[0]
        new_time_array = utils.create_time_series(sampling_frequency=new_sampling_frequency,
                                                  duration=new_duration,
                                                  starting_time=new_start_time)
        self.assertTrue(np.allclose(self.time_array, new_time_array))

    def test_consistency_frequency_array_to_frequency_array(self):
        new_sampling_frequency, new_duration = utils.get_sampling_frequency_and_duration_from_frequency_array(self.frequency_array)
        new_frequency_array = \
            utils.create_frequency_series(sampling_frequency=new_sampling_frequency,
                                          duration=new_duration)
        self.assertTrue(np.allclose(self.frequency_array, new_frequency_array))

    def test_illegal_sampling_frequency_and_duration(self):
        with self.assertRaises(utils.IllegalDurationAndSamplingFrequencyException):
            _ = utils.create_time_series(sampling_frequency=7.7,
                                         duration=1.3,
                                         starting_time=0)


class TestReflect(unittest.TestCase):

    def test_in_range(self):
        xprime = np.array([0.1, 0.5, 0.9])
        x = np.array([0.1, 0.5, 0.9])
        self.assertTrue(
            np.testing.assert_allclose(utils.reflect(xprime),  x) is None)

    def test_in_one_to_two(self):
        xprime = np.array([1.1, 1.5, 1.9])
        x = np.array([0.9, 0.5, 0.1])
        self.assertTrue(
            np.testing.assert_allclose(utils.reflect(xprime),  x) is None)

    def test_in_two_to_three(self):
        xprime = np.array([2.1, 2.5, 2.9])
        x = np.array([0.1, 0.5, 0.9])
        self.assertTrue(
            np.testing.assert_allclose(utils.reflect(xprime),  x) is None)

    def test_in_minus_one_to_zero(self):
        xprime = np.array([-0.9, -0.5, -0.1])
        x = np.array([0.9, 0.5, 0.1])
        self.assertTrue(
            np.testing.assert_allclose(utils.reflect(xprime),  x) is None)

    def test_in_minus_two_to_minus_one(self):
        xprime = np.array([-1.9, -1.5, -1.1])
        x = np.array([0.1, 0.5, 0.9])
        self.assertTrue(
            np.testing.assert_allclose(utils.reflect(xprime),  x) is None)

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
import bilby
from bilby.core import prior
import unittest
from mock import MagicMock
import numpy as np
import os
import shutil
import copy


class TestSampler(unittest.TestCase):

    def setUp(self):
        likelihood = bilby.core.likelihood.Likelihood()
        likelihood.parameters = dict(a=1, b=2, c=3)
        delta_prior = prior.DeltaFunction(peak=0)
        delta_prior.rescale = MagicMock(return_value=prior.DeltaFunction(peak=1))
        delta_prior.prob = MagicMock(return_value=1)
        delta_prior.sample = MagicMock(return_value=0)
        uniform_prior = prior.Uniform(0, 1)
        uniform_prior.rescale = MagicMock(return_value=prior.Uniform(0, 2))
        uniform_prior.prob = MagicMock(return_value=1)
        uniform_prior.sample = MagicMock(return_value=0.5)

        priors = dict(a=delta_prior, b='string', c=uniform_prior)
        likelihood.log_likelihood_ratio = MagicMock(return_value=1)
        likelihood.log_likelihood = MagicMock(return_value=2)
        test_directory = 'test_directory'
        if os.path.isdir(test_directory):
            os.rmdir(test_directory)
        self.sampler = bilby.core.sampler.Sampler(
            likelihood=likelihood, priors=priors,
            outdir=test_directory, use_ratio=False,
            skip_import_verification=True)

    def tearDown(self):
        del self.sampler

    def test_search_parameter_keys(self):
        expected_search_parameter_keys = ['c']
        self.assertListEqual(self.sampler.search_parameter_keys, expected_search_parameter_keys)

    def test_fixed_parameter_keys(self):
        expected_fixed_parameter_keys = ['a']
        self.assertListEqual(self.sampler.fixed_parameter_keys, expected_fixed_parameter_keys)

    def test_ndim(self):
        self.assertEqual(self.sampler.ndim, 1)

    def test_kwargs(self):
        self.assertDictEqual(self.sampler.kwargs, {})

    def test_label(self):
        self.assertEqual(self.sampler.label, 'label')

    def test_prior_transform_transforms_search_parameter_keys(self):
        self.sampler.prior_transform([0])
        expected_prior = prior.Uniform(0, 1)
        self.assertListEqual([self.sampler.priors['c'].minimum,
                              self.sampler.priors['c'].maximum],
                             [expected_prior.minimum,
                              expected_prior.maximum])

    def test_prior_transform_does_not_transform_fixed_parameter_keys(self):
        self.sampler.prior_transform([0])
        self.assertEqual(self.sampler.priors['a'].peak,
                         prior.DeltaFunction(peak=0).peak)

    def test_log_prior(self):
        self.assertEqual(self.sampler.log_prior({1}), 0.0)

    def test_log_likelihood_with_use_ratio(self):
        self.sampler.use_ratio = True
        self.assertEqual(self.sampler.log_likelihood([0]), 1)

    def test_log_likelihood_without_use_ratio(self):
        self.sampler.use_ratio = False
        self.assertEqual(self.sampler.log_likelihood([0]), 2)

    def test_log_likelihood_correctly_sets_parameters(self):
        expected_dict = dict(a=0,
                             b=2,
                             c=0)
        _ = self.sampler.log_likelihood([0])
        self.assertDictEqual(self.sampler.likelihood.parameters, expected_dict)

    def test_get_random_draw(self):
        self.assertEqual(self.sampler.get_random_draw_from_prior(), np.array([0.5]))

    def test_base_run_sampler(self):
        sampler_copy = copy.copy(self.sampler)
        self.sampler.run_sampler()
        self.assertDictEqual(sampler_copy.__dict__, self.sampler.__dict__)


class TestCPNest(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Cpnest(self.likelihood, self.priors,
                                                 outdir='outdir', label='label',
                                                 use_ratio=False, plot=False,
                                                 skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(verbose=3, nthreads=1, nlive=500, maxmcmc=1000,
                        seed=None, poolsize=100, nhamiltonian=0, resume=True,
                        output='outdir/cpnest_label/', proposals=None,
                        n_periodic_checkpoint=8000)
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(verbose=3, nthreads=1, nlive=250, maxmcmc=1000,
                        seed=None, poolsize=100, nhamiltonian=0, resume=True,
                        output='outdir/cpnest_label/', proposals=None,
                        n_periodic_checkpoint=8000)
        for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['nlive']
            new_kwargs[equiv] = 250
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestDynesty(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Dynesty(self.likelihood, self.priors,
                                                  outdir='outdir', label='label',
                                                  use_ratio=False, plot=False,
                                                  skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(bound='multi', sample='rwalk', periodic=None, reflective=None, verbose=True,
                        check_point_delta_t=600, nlive=1000, first_update=None,
                        npdim=None, rstate=None, queue_size=None, pool=None,
                        use_pool=None, live_points=None, logl_args=None, logl_kwargs=None,
                        ptform_args=None, ptform_kwargs=None,
                        enlarge=1.5, bootstrap=None, vol_dec=0.5, vol_check=8.0,
                        facc=0.2, slices=5, dlogz=0.1, maxiter=None, maxcall=None,
                        logl_max=np.inf, add_live=True, print_progress=True, save_bounds=False,
                        walks=100, update_interval=600, print_func='func', n_effective=None,
                        maxmcmc=5000, nact=5)
        self.sampler.kwargs['print_func'] = 'func'  # set this manually as this is not testable otherwise
        # DictEqual can't handle lists so we check these separately
        self.assertEqual([], self.sampler.kwargs['periodic'])
        self.assertEqual([], self.sampler.kwargs['reflective'])
        self.sampler.kwargs['periodic'] = expected['periodic']
        self.sampler.kwargs['reflective'] = expected['reflective']
        for key in self.sampler.kwargs.keys():
            print("key={}, expected={}, actual={}"
                  .format(key, expected[key], self.sampler.kwargs[key]))
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(bound='multi', sample='rwalk', periodic=[], reflective=[], verbose=True,
                        check_point_delta_t=600, nlive=1000, first_update=None,
                        npdim=None, rstate=None, queue_size=None, pool=None,
                        use_pool=None, live_points=None, logl_args=None, logl_kwargs=None,
                        ptform_args=None, ptform_kwargs=None,
                        enlarge=1.5, bootstrap=None, vol_dec=0.5, vol_check=8.0,
                        facc=0.2, slices=5, dlogz=0.1, maxiter=None, maxcall=None,
                        logl_max=np.inf, add_live=True, print_progress=True, save_bounds=False,
                        walks=100, update_interval=600, print_func='func', n_effective=None,
                        maxmcmc=5000, nact=5)

        for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['nlive']
            new_kwargs[equiv] = 1000
            self.sampler.kwargs = new_kwargs
            self.sampler.kwargs['print_func'] = 'func'  # set this manually as this is not testable otherwise
            self.assertDictEqual(expected, self.sampler.kwargs)

    def test_prior_boundary(self):
        self.priors['a'] = bilby.core.prior.Prior(boundary='periodic')
        self.priors['b'] = bilby.core.prior.Prior(boundary='reflective')
        self.priors['c'] = bilby.core.prior.Prior(boundary=None)
        self.priors['d'] = bilby.core.prior.Prior(boundary='reflective')
        self.priors['e'] = bilby.core.prior.Prior(boundary='periodic')
        self.sampler = bilby.core.sampler.Dynesty(self.likelihood, self.priors,
                                                  outdir='outdir', label='label',
                                                  use_ratio=False, plot=False,
                                                  skip_import_verification=True)
        self.assertEqual([0, 4], self.sampler.kwargs["periodic"])
        self.assertEqual(self.sampler._periodic, self.sampler.kwargs["periodic"])
        self.assertEqual([1, 3], self.sampler.kwargs["reflective"])
        self.assertEqual(self.sampler._reflective, self.sampler.kwargs["reflective"])


class TestEmcee(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Emcee(self.likelihood, self.priors,
                                                outdir='outdir', label='label',
                                                use_ratio=False, plot=False,
                                                skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(nwalkers=500, a=2, args=[], kwargs={},
                        postargs=None, pool=None, live_dangerously=False,
                        runtime_sortingfn=None, lnprob0=None, rstate0=None,
                        blobs0=None, iterations=100, thin=1, storechain=True, mh_proposal=None
                        )
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(nwalkers=100, a=2, args=[], kwargs={},
                        postargs=None, pool=None, live_dangerously=False,
                        runtime_sortingfn=None, lnprob0=None, rstate0=None,
                        blobs0=None, iterations=100, thin=1, storechain=True, mh_proposal=None)
        for equiv in bilby.core.sampler.base_sampler.MCMCSampler.nwalkers_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['nwalkers']
            new_kwargs[equiv] = 100
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestKombine(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Kombine(self.likelihood, self.priors,
                                                  outdir='outdir', label='label',
                                                  use_ratio=False, plot=False,
                                                  skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(nwalkers=500, args=[], pool=None, transd=False,
                        lnpost0=None, blob0=None, iterations=500, storechain=True, processes=1, update_interval=None,
                        kde=None, kde_size=None, spaces=None, freeze_transd=False, test_steps=16, critical_pval=0.05,
                        max_steps=None, burnin_verbose=False)
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(nwalkers=400, args=[], pool=None, transd=False,
                        lnpost0=None, blob0=None, iterations=500, storechain=True, processes=1, update_interval=None,
                        kde=None, kde_size=None, spaces=None, freeze_transd=False, test_steps=16, critical_pval=0.05,
                        max_steps=None, burnin_verbose=False)
        for equiv in bilby.core.sampler.base_sampler.MCMCSampler.nwalkers_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['nwalkers']
            new_kwargs[equiv] = 400
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestNestle(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Nestle(self.likelihood, self.priors,
                                                 outdir='outdir', label='label',
                                                 use_ratio=False, plot=False,
                                                 skip_import_verification=True,
                                                 verbose=False)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(verbose=False, method='multi', npoints=500,
                        update_interval=None, npdim=None, maxiter=None,
                        maxcall=None, dlogz=None, decline_factor=None,
                        rstate=None, callback=None, steps=20, enlarge=1.2)
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(verbose=False, method='multi', npoints=345,
                        update_interval=None, npdim=None, maxiter=None,
                        maxcall=None, dlogz=None, decline_factor=None,
                        rstate=None, callback=None, steps=20, enlarge=1.2)
        self.sampler.kwargs['npoints'] = 123
        for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['npoints']
            new_kwargs[equiv] = 345
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestPolyChord(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.PyPolyChord(self.likelihood, self.priors,
                                                      outdir='outdir', label='polychord',
                                                      use_ratio=False, plot=False,
                                                      skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(use_polychord_defaults=False, nlive=self.sampler.ndim*25, num_repeats=self.sampler.ndim*5,
                        nprior=-1, do_clustering=True, feedback=1, precision_criterion=0.001,
                        logzero=-1e30, max_ndead=-1, boost_posterior=0.0, posteriors=True,
                        equals=True, cluster_posteriors=True, write_resume=True,
                        write_paramnames=False, read_resume=True, write_stats=True,
                        write_live=True, write_dead=True, write_prior=True,
                        compression_factor=np.exp(-1), base_dir='outdir',
                        file_root='polychord', seed=-1, grade_dims=list([self.sampler.ndim]),
                        grade_frac=list([1.0]*len([self.sampler.ndim])), nlives={})
        self.sampler._setup_dynamic_defaults()
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(use_polychord_defaults=False, nlive=123, num_repeats=self.sampler.ndim*5,
                        nprior=-1, do_clustering=True, feedback=1, precision_criterion=0.001,
                        logzero=-1e30, max_ndead=-1, boost_posterior=0.0, posteriors=True,
                        equals=True, cluster_posteriors=True, write_resume=True,
                        write_paramnames=False, read_resume=True, write_stats=True,
                        write_live=True, write_dead=True, write_prior=True,
                        compression_factor=np.exp(-1), base_dir='outdir',
                        file_root='polychord', seed=-1, grade_dims=list([self.sampler.ndim]),
                        grade_frac=list([1.0]*len([self.sampler.ndim])), nlives={})
        self.sampler._setup_dynamic_defaults()
        for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['nlive']
            new_kwargs[equiv] = 123
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestPTEmcee(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Ptemcee(self.likelihood, self.priors,
                                                  outdir='outdir', label='label',
                                                  use_ratio=False, plot=False,
                                                  skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(ntemps=2, nwalkers=500,
                        Tmax=None, betas=None,
                        threads=1, pool=None, a=2.0,
                        loglargs=[], logpargs=[],
                        loglkwargs={}, logpkwargs={},
                        adaptation_lag=10000, adaptation_time=100,
                        random=None, iterations=100, thin=1,
                        storechain=True, adapt=True,
                        swap_ratios=False,
                        )
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(ntemps=2, nwalkers=150,
                        Tmax=None, betas=None,
                        threads=1, pool=None, a=2.0,
                        loglargs=[], logpargs=[],
                        loglkwargs={}, logpkwargs={},
                        adaptation_lag=10000, adaptation_time=100,
                        random=None, iterations=100, thin=1,
                        storechain=True, adapt=True,
                        swap_ratios=False,
                        )
        for equiv in bilby.core.sampler.base_sampler.MCMCSampler.nwalkers_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['nwalkers']
            new_kwargs[equiv] = 150
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestPyMC3(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.sampler = bilby.core.sampler.Pymc3(self.likelihood, self.priors,
                                                outdir='outdir', label='label',
                                                use_ratio=False, plot=False,
                                                skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(
            draws=500, step=None, init='auto', n_init=200000, start=None, trace=None, chain_idx=0,
            chains=2, cores=1, tune=500, nuts_kwargs=None, step_kwargs=None, progressbar=True,
            model=None, random_seed=None, discard_tuned_samples=True,
            compute_convergence_checks=True)
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(
            draws=500, step=None, init='auto', n_init=200000, start=None, trace=None, chain_idx=0,
            chains=2, cores=1, tune=500, nuts_kwargs=None, step_kwargs=None, progressbar=True,
            model=None, random_seed=None, discard_tuned_samples=True,
            compute_convergence_checks=True)
        self.sampler.kwargs['draws'] = 123
        for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['draws']
            new_kwargs[equiv] = 500
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestPymultinest(unittest.TestCase):

    def setUp(self):
        self.likelihood = MagicMock()
        self.priors = bilby.core.prior.PriorDict(
            dict(a=bilby.core.prior.Uniform(0, 1),
                 b=bilby.core.prior.Uniform(0, 1)))
        self.priors['a'] = bilby.core.prior.Prior(boundary='periodic')
        self.priors['b'] = bilby.core.prior.Prior(boundary='reflective')
        self.sampler = bilby.core.sampler.Pymultinest(self.likelihood, self.priors,
                                                      outdir='outdir', label='label',
                                                      use_ratio=False, plot=False,
                                                      skip_import_verification=True)

    def tearDown(self):
        del self.likelihood
        del self.priors
        del self.sampler

    def test_default_kwargs(self):
        expected = dict(importance_nested_sampling=False, resume=True,
                        verbose=True, sampling_efficiency='parameter',
                        outputfiles_basename='outdir/pm_label/',
                        n_live_points=500, n_params=2,
                        n_clustering_params=None, wrapped_params=None,
                        multimodal=True, const_efficiency_mode=False,
                        evidence_tolerance=0.5,
                        n_iter_before_update=100, null_log_evidence=-1e90,
                        max_modes=100, mode_tolerance=-1e90, seed=-1,
                        context=0, write_output=True, log_zero=-1e100,
                        max_iter=0, init_MPI=False, dump_callback=None)
        self.assertListEqual([1, 0], self.sampler.kwargs['wrapped_params'])  # Check this separately
        self.sampler.kwargs['wrapped_params'] = None  # The dict comparison can't handle lists
        self.assertDictEqual(expected, self.sampler.kwargs)

    def test_translate_kwargs(self):
        expected = dict(importance_nested_sampling=False, resume=True,
                        verbose=True, sampling_efficiency='parameter',
                        outputfiles_basename='outdir/pm_label/',
                        n_live_points=123, n_params=2,
                        n_clustering_params=None, wrapped_params=None,
                        multimodal=True, const_efficiency_mode=False,
                        evidence_tolerance=0.5,
                        n_iter_before_update=100, null_log_evidence=-1e90,
                        max_modes=100, mode_tolerance=-1e90, seed=-1,
                        context=0, write_output=True, log_zero=-1e100,
                        max_iter=0, init_MPI=False, dump_callback=None)
        for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
            new_kwargs = self.sampler.kwargs.copy()
            del new_kwargs['n_live_points']
            new_kwargs['wrapped_params'] = None  # The dict comparison can't handle lists
            new_kwargs[equiv] = 123
            self.sampler.kwargs = new_kwargs
            self.assertDictEqual(expected, self.sampler.kwargs)


class TestRunningSamplers(unittest.TestCase):

    def setUp(self):
        np.random.seed(42)
        bilby.core.utils.command_line_args.bilby_test_mode = False
        self.x = np.linspace(0, 1, 11)
        self.model = lambda x, m, c: m * x + c
        self.injection_parameters = dict(m=0.5, c=0.2)
        self.sigma = 0.1
        self.y = self.model(self.x, **self.injection_parameters) +\
            np.random.normal(0, self.sigma, len(self.x))
        self.likelihood = bilby.likelihood.GaussianLikelihood(
            self.x, self.y, self.model, self.sigma)

        self.priors = bilby.core.prior.PriorDict()
        self.priors['m'] = bilby.core.prior.Uniform(0, 5, boundary='reflective')
        self.priors['c'] = bilby.core.prior.Uniform(-2, 2, boundary='reflective')
        bilby.core.utils.check_directory_exists_and_if_not_mkdir('outdir')

    def tearDown(self):
        del self.likelihood
        del self.priors
        bilby.core.utils.command_line_args.bilby_test_mode = False
        shutil.rmtree('outdir')

    def test_run_cpnest(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='cpnest',
            nlive=100, save=False, resume=False)

    def test_run_dynesty(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='dynesty',
            nlive=100, save=False)

    def test_run_dynamic_dynesty(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='dynamic_dynesty',
            nlive_init=100, nlive_batch=100, dlogz_init=1.0, maxbatch=0,
            maxcall=100, bound="single", save=False)

    def test_run_emcee(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='emcee',
            iterations=1000, nwalkers=10, save=False)

    def test_run_kombine(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='kombine',
            iterations=1000, nwalkers=100, save=False, autoburnin=True)

    def test_run_nestle(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='nestle',
            nlive=100, save=False)

    def test_run_pypolychord(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors,
            sampler='pypolychord', nlive=100, save=False)

    def test_run_ptemcee(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='ptemcee',
            nsteps=1000, nwalkers=10, ntemps=10, save=False)

    def test_run_pymc3(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors, sampler='pymc3',
            draws=50, tune=50, n_init=1000, save=False)

    def test_run_pymultinest(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors,
            sampler='pymultinest', nlive=100, save=False)

    def test_run_PTMCMCSampler(self):
        _ = bilby.run_sampler(
            likelihood=self.likelihood, priors=self.priors,
            sampler='PTMCMCsampler', Niter=101, burn=2,
            isave=100, save=False)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division

import unittest
import numpy as np
import pandas as pd
import shutil
import os
import json
from scipy.stats import multivariate_normal

import bilby


# set 2D multivariate Gaussian likelihood
class MultiGaussian(bilby.Likelihood):
    def __init__(self, mean, cov):
        super(MultiGaussian, self).__init__(parameters=dict())
        self.cov = np.array(cov)
        self.mean = np.array(mean)
        self.sigma = np.sqrt(np.diag(self.cov))
        self.pdf = multivariate_normal(mean=self.mean, cov=self.cov)

    @property
    def dim(self):
        return len(self.cov[0])

    def log_likelihood(self):
        x = np.array([self.parameters["x{0}".format(i)] for i in range(self.dim)])
        return self.pdf.logpdf(x)


class TestGrid(unittest.TestCase):

    def setUp(self):
        np.random.seed(7)
        
        # set 2D multivariate Gaussian (zero mean, unit variance)
        self.mus = [0., 0.]
        self.cov = [[1., 0.], [0., 1.]]
        dim = len(self.mus)
        self.likelihood = MultiGaussian(self.mus, self.cov)
        
        # set priors out to +/- 5 sigma
        self.priors = bilby.core.prior.PriorDict()
        self.priors.update(
            {"x{0}".format(i): bilby.core.prior.Uniform(-5, 5, "x{0}".format(i)) for i in range(dim)}
        )

        # expected evidence integral should be (1/V) where V is the prior volume
        log_prior_vol = np.sum(np.log([prior.maximum - prior.minimum for key, prior in self.priors.items()]))
        self.expected_ln_evidence = -log_prior_vol

        self.grid_size = 100

        grid = bilby.core.grid.Grid(
            label='label', outdir='outdir', priors=self.priors,
            grid_size=self.grid_size, likelihood=self.likelihood,
            save=True
        )

        self.grid = grid
        pass

    def tearDown(self):
        bilby.utils.command_line_args.bilby_test_mode = True
        try:
            shutil.rmtree(self.grid.outdir)
        except OSError:
            pass
        del self.grid
        pass

    def test_grid_file_name_default(self):
        outdir = 'outdir'
        label = 'label'
        self.assertEqual(bilby.core.grid.grid_file_name(outdir, label),
                         '{}/{}_grid.json'.format(outdir, label))
        self.assertEqual(bilby.core.grid.grid_file_name(outdir, label, True),
                         '{}/{}_grid.json.gz'.format(outdir, label))

    def test_fail_save_and_load(self):
        with self.assertRaises(ValueError):
            bilby.core.grid.Grid.read()

        with self.assertRaises(IOError):
            bilby.core.grid.Grid.read(filename='not/a/file.json')

    def test_fail_marginalize(self):
        with self.assertRaises(TypeError):
            self.grid.marginalize_posterior(parameters=2.4)
        
        with self.assertRaises(TypeError):
            self.grid.marginalize_posterior(not_parameters=4.7)
        
        with self.assertRaises(ValueError):
            self.grid.marginalize_posterior(parameters='jkgsd')

    def test_parameter_names(self):
        assert list(self.priors.keys()) == self.grid.parameter_names
        assert self.grid.n_dims == 2

    def test_no_marginalization(self):
        # test arrays are the same if no parameters are given to marginalize
        # over
        assert np.array_equal(self.grid.ln_likelihood,
                              self.grid.marginalize_ln_likelihood(not_parameters=self.grid.parameter_names))
    
    def test_marginalization_shapes(self):
        assert len(self.grid.marginalize_ln_likelihood().shape) == 0
        
        marg1 = self.grid.marginalize_ln_likelihood(parameters=self.grid.parameter_names[0])
        assert marg1.shape == (self.grid_size,)

        marg2 = self.grid.marginalize_ln_likelihood(parameters=self.grid.parameter_names[1])
        assert marg2.shape == (self.grid_size,)

        assert self.grid.ln_likelihood.shape == (self.grid_size, self.grid_size)
        assert self.grid.ln_posterior.shape == (self.grid_size, self.grid_size)

    def test_marginalization_opposite(self):
        assert np.array_equal(self.grid.marginalize_ln_likelihood(parameters=self.grid.parameter_names[0]),
                              self.grid.marginalize_ln_likelihood(not_parameters=self.grid.parameter_names[1]))
        assert np.array_equal(self.grid.marginalize_ln_likelihood(parameters=self.grid.parameter_names[1]),
                              self.grid.marginalize_ln_likelihood(not_parameters=self.grid.parameter_names[0]))
    
    def test_max_marginalized_likelihood(self):
        # marginalised likelihoods should have max values of 1 (as they are not
        # properly normalised)
        assert self.grid.marginalize_likelihood(self.grid.parameter_names[0]).max() == 1.
        assert self.grid.marginalize_likelihood(self.grid.parameter_names[1]).max() == 1.

    def test_ln_evidence(self):
        assert np.isclose(self.grid.ln_evidence, self.expected_ln_evidence)

    def test_fail_grid_size(self):
        with self.assertRaises(TypeError):
            grid = bilby.core.grid.Grid(
                label='label', outdir='outdir', priors=self.priors,
                grid_size=2.3, likelihood=self.likelihood,
                save=True
            )

    def test_mesh_grid(self):
        assert self.grid.mesh_grid[0].shape == (self.grid_size, self.grid_size)
        assert self.grid.mesh_grid[0][0,0] == self.priors[self.grid.parameter_names[0]].minimum
        assert self.grid.mesh_grid[0][-1,-1] == self.priors[self.grid.parameter_names[1]].maximum

    def test_different_grids(self):
        npoints = [10, 20]

        grid = bilby.core.grid.Grid(
            label='label', outdir='outdir', priors=self.priors,
            grid_size=npoints, likelihood=self.likelihood
        )

        assert grid.mesh_grid[0].shape == tuple(npoints)
        assert grid.mesh_grid[0][0,0] == self.priors[self.grid.parameter_names[0]].minimum
        assert grid.mesh_grid[0][-1,-1] == self.priors[self.grid.parameter_names[1]].maximum

        del grid

        npoints = {'x0': 15, 'x1': 18}

        grid = bilby.core.grid.Grid(
            label='label', outdir='outdir', priors=self.priors,
            grid_size=npoints, likelihood=self.likelihood
        )

        assert grid.mesh_grid[0].shape == (npoints['x0'], npoints['x1'])
        assert grid.mesh_grid[0][0,0] == self.priors[self.grid.parameter_names[0]].minimum
        assert grid.mesh_grid[0][-1,-1] == self.priors[self.grid.parameter_names[1]].maximum

        del grid

        x0s = np.linspace(self.priors['x0'].minimum, self.priors['x0'].maximum, 13)
        x1s = np.linspace(self.priors['x0'].minimum, self.priors['x0'].maximum, 14)
        npoints = {'x0': x0s,
                   'x1': x1s}

        grid = bilby.core.grid.Grid(
            label='label', outdir='outdir', priors=self.priors,
            grid_size=npoints, likelihood=self.likelihood
        )

        assert grid.mesh_grid[0].shape == (len(x0s), len(x1s))
        assert grid.mesh_grid[0][0,0] == self.priors[self.grid.parameter_names[0]].minimum
        assert grid.mesh_grid[0][-1,-1] == self.priors[self.grid.parameter_names[1]].maximum
        assert np.array_equal(grid.sample_points['x0'], x0s)
        assert np.array_equal(grid.sample_points['x1'], x1s)

    def test_save_and_load(self):
        filename = os.path.join('outdir', 'test_output.json')

        self.grid.save_to_file(filename=filename)

        # load file
        newgrid = bilby.core.grid.Grid.read(filename=filename)

        assert newgrid.parameter_names == self.grid.parameter_names
        assert newgrid.n_dims == self.grid.n_dims
        assert np.array_equal(newgrid.mesh_grid[0], self.grid.mesh_grid[0])
        for par in newgrid.parameter_names:
            assert np.array_equal(newgrid.sample_points[par], self.grid.sample_points[par])
        assert newgrid.ln_evidence == self.grid.ln_evidence
        assert np.array_equal(newgrid.ln_likelihood, self.grid.ln_likelihood)
        assert np.array_equal(newgrid.ln_posterior, self.grid.ln_posterior)

        del newgrid

        self.grid.save_to_file(overwrite=True, outdir='outdir')

        # load file
        newgrid = bilby.core.grid.Grid.read(outdir='outdir',
                                            label='label')

        assert newgrid.parameter_names == self.grid.parameter_names
        assert newgrid.n_dims == self.grid.n_dims
        assert np.array_equal(newgrid.mesh_grid[0], self.grid.mesh_grid[0])
        for par in newgrid.parameter_names:
            assert np.array_equal(newgrid.sample_points[par], self.grid.sample_points[par])
        assert newgrid.ln_evidence == self.grid.ln_evidence
        assert np.array_equal(newgrid.ln_likelihood, self.grid.ln_likelihood)
        assert np.array_equal(newgrid.ln_posterior, self.grid.ln_posterior)

        del newgrid

    def test_save_and_load_gzip(self):
        filename = os.path.join('outdir', 'test_output.json.gz')

        self.grid.save_to_file(filename=filename)

        # load file
        newgrid = bilby.core.grid.Grid.read(filename=filename)

        assert newgrid.parameter_names == self.grid.parameter_names
        assert newgrid.n_dims == self.grid.n_dims
        assert np.array_equal(newgrid.mesh_grid[0], self.grid.mesh_grid[0])
        for par in newgrid.parameter_names:
            assert np.array_equal(newgrid.sample_points[par], self.grid.sample_points[par])
        assert newgrid.ln_evidence == self.grid.ln_evidence
        assert np.array_equal(newgrid.ln_likelihood, self.grid.ln_likelihood)
        assert np.array_equal(newgrid.ln_posterior, self.grid.ln_posterior)

        del newgrid

        self.grid.save_to_file(overwrite=True, outdir='outdir',
                               gzip=True)

        # load file
        newgrid = bilby.core.grid.Grid.read(outdir='outdir', label='label',
                                            gzip=True)

        assert newgrid.parameter_names == self.grid.parameter_names
        assert newgrid.n_dims == self.grid.n_dims
        assert np.array_equal(newgrid.mesh_grid[0], self.grid.mesh_grid[0])
        for par in newgrid.parameter_names:
            assert np.array_equal(newgrid.sample_points[par], self.grid.sample_points[par])
        assert newgrid.ln_evidence == self.grid.ln_evidence
        assert np.array_equal(newgrid.ln_likelihood, self.grid.ln_likelihood)
        assert np.array_equal(newgrid.ln_posterior, self.grid.ln_posterior)

        del newgrid



from __future__ import division, absolute_import
import unittest
import mock

import numpy as np

import bilby
from bilby.gw import conversion


class TestBasicConversions(unittest.TestCase):

    def setUp(self):
        self.mass_1 = 1.4
        self.mass_2 = 1.3
        self.mass_ratio = 13/14
        self.total_mass = 2.7
        self.chirp_mass = (1.4 * 1.3)**0.6 / 2.7**0.2
        self.symmetric_mass_ratio = (1.4 * 1.3) / 2.7**2
        self.cos_angle = -1
        self.angle = np.pi
        self.lambda_1 = 300
        self.lambda_2 = 300 * (14 / 13)**5
        self.lambda_tilde = 8 / 13 * (
            (1 + 7 * self.symmetric_mass_ratio
             - 31 * self.symmetric_mass_ratio**2)
            * (self.lambda_1 + self.lambda_2)
            + (1 - 4 * self.symmetric_mass_ratio)**0.5
            * (1 + 9 * self.symmetric_mass_ratio
               - 11 * self.symmetric_mass_ratio**2)
            * (self.lambda_1 - self.lambda_2)
        )
        self.delta_lambda_tilde = 1 / 2 * (
                (1 - 4 * self.symmetric_mass_ratio)**0.5
                * (1 - 13272 / 1319 * self.symmetric_mass_ratio
                   + 8944 / 1319 * self.symmetric_mass_ratio**2)
                * (self.lambda_1 + self.lambda_2)
                + (1 - 15910 / 1319 * self.symmetric_mass_ratio
                   + 32850 / 1319 * self.symmetric_mass_ratio**2
                   + 3380 / 1319 * self.symmetric_mass_ratio**3)
                * (self.lambda_1 - self.lambda_2)
        )

    def tearDown(self):
        del self.mass_1
        del self.mass_2
        del self.mass_ratio
        del self.total_mass
        del self.chirp_mass
        del self.symmetric_mass_ratio

    def test_total_mass_and_mass_ratio_to_component_masses(self):
        mass_1, mass_2 = conversion.total_mass_and_mass_ratio_to_component_masses(self.mass_ratio, self.total_mass)
        self.assertTrue(all([abs(mass_1 - self.mass_1) < 1e-5,
                             abs(mass_2 - self.mass_2) < 1e-5]))

    def test_chirp_mass_and_primary_mass_to_mass_ratio(self):
        mass_ratio = conversion.chirp_mass_and_primary_mass_to_mass_ratio(
            self.chirp_mass, self.mass_1)
        self.assertAlmostEqual(self.mass_ratio, mass_ratio)

    def test_symmetric_mass_ratio_to_mass_ratio(self):
        mass_ratio = conversion.symmetric_mass_ratio_to_mass_ratio(self.symmetric_mass_ratio)
        self.assertAlmostEqual(self.mass_ratio, mass_ratio)

    def test_chirp_mass_and_total_mass_to_symmetric_mass_ratio(self):
        symmetric_mass_ratio = conversion.chirp_mass_and_total_mass_to_symmetric_mass_ratio(self.chirp_mass, self.total_mass)
        self.assertAlmostEqual(self.symmetric_mass_ratio, symmetric_mass_ratio)

    def test_chirp_mass_and_mass_ratio_to_total_mass(self):
        total_mass = conversion.chirp_mass_and_mass_ratio_to_total_mass(self.chirp_mass, self.mass_ratio)
        self.assertAlmostEqual(self.total_mass, total_mass)

    def test_component_masses_to_chirp_mass(self):
        chirp_mass = conversion.component_masses_to_chirp_mass(self.mass_1, self.mass_2)
        self.assertAlmostEqual(self.chirp_mass, chirp_mass)

    def test_component_masses_to_total_mass(self):
        total_mass = conversion.component_masses_to_total_mass(self.mass_1, self.mass_2)
        self.assertAlmostEqual(self.total_mass, total_mass)

    def test_component_masses_to_symmetric_mass_ratio(self):
        symmetric_mass_ratio = conversion.component_masses_to_symmetric_mass_ratio(self.mass_1, self.mass_2)
        self.assertAlmostEqual(self.symmetric_mass_ratio, symmetric_mass_ratio)

    def test_component_masses_to_mass_ratio(self):
        mass_ratio = conversion.component_masses_to_mass_ratio(self.mass_1, self.mass_2)
        self.assertAlmostEqual(self.mass_ratio, mass_ratio)

    def test_mass_1_and_chirp_mass_to_mass_ratio(self):
        mass_ratio = conversion.mass_1_and_chirp_mass_to_mass_ratio(self.mass_1, self.chirp_mass)
        self.assertAlmostEqual(self.mass_ratio, mass_ratio)

    def test_lambda_tilde_to_lambda_1_lambda_2(self):
        lambda_1, lambda_2 =\
            conversion.lambda_tilde_to_lambda_1_lambda_2(
                self.lambda_tilde, self.mass_1, self.mass_2)
        self.assertTrue(all([abs(self.lambda_1 - lambda_1) < 1e-5,
                             abs(self.lambda_2 - lambda_2) < 1e-5]))

    def test_lambda_tilde_delta_lambda_tilde_to_lambda_1_lambda_2(self):
        lambda_1, lambda_2 =\
            conversion.lambda_tilde_delta_lambda_tilde_to_lambda_1_lambda_2(
                self.lambda_tilde, self.delta_lambda_tilde, self.mass_1, self.mass_2)
        self.assertTrue(all([abs(self.lambda_1 - lambda_1) < 1e-5,
                             abs(self.lambda_2 - lambda_2) < 1e-5]))

    def test_lambda_1_lambda_2_to_lambda_tilde(self):
        lambda_tilde = \
            conversion.lambda_1_lambda_2_to_lambda_tilde(
                self.lambda_1, self.lambda_2, self.mass_1, self.mass_2)
        self.assertTrue((self.lambda_tilde - lambda_tilde) < 1e-5)

    def test_lambda_1_lambda_2_to_delta_lambda_tilde(self):
        delta_lambda_tilde = \
            conversion.lambda_1_lambda_2_to_delta_lambda_tilde(
                self.lambda_1, self.lambda_2, self.mass_1, self.mass_2)
        self.assertTrue((self.delta_lambda_tilde - delta_lambda_tilde) < 1e-5)


class TestConvertToLALParams(unittest.TestCase):

    def setUp(self):
        self.search_keys = []
        self.parameters = dict()
        self.component_mass_pars = dict(mass_1=1.4, mass_2=1.4)
        self.mass_parameters = self.component_mass_pars.copy()
        self.mass_parameters['mass_ratio'] =\
            conversion.component_masses_to_mass_ratio(**self.component_mass_pars)
        self.mass_parameters['symmetric_mass_ratio'] = \
            conversion.component_masses_to_symmetric_mass_ratio(**self.component_mass_pars)
        self.mass_parameters['chirp_mass'] = \
            conversion.component_masses_to_chirp_mass(**self.component_mass_pars)
        self.mass_parameters['total_mass'] = \
            conversion.component_masses_to_total_mass(**self.component_mass_pars)
        self.component_tidal_parameters = dict(lambda_1=300, lambda_2=300)
        self.all_component_pars = self.component_tidal_parameters.copy()
        self.all_component_pars.update(self.component_mass_pars)
        self.tidal_parameters = self.component_tidal_parameters.copy()
        self.tidal_parameters['lambda_tilde'] = \
            conversion.lambda_1_lambda_2_to_lambda_tilde(**self.all_component_pars)
        self.tidal_parameters['delta_lambda_tilde'] = \
            conversion.lambda_1_lambda_2_to_delta_lambda_tilde(**self.all_component_pars)

    def tearDown(self):
        del self.search_keys
        del self.parameters

    def bbh_convert(self):
        self.parameters, self.added_keys =\
            conversion.convert_to_lal_binary_black_hole_parameters(
                self.parameters)

    def bns_convert(self):
        self.parameters, self.added_keys = \
            conversion.convert_to_lal_binary_neutron_star_parameters(
                self.parameters)

    def test_redshift_to_luminosity_distance(self):
        self.parameters['redshift'] = 1
        dl = conversion.redshift_to_luminosity_distance(
            self.parameters['redshift'])
        self.bbh_convert()
        self.assertEqual(self.parameters['luminosity_distance'], dl)

    def test_comoving_to_luminosity_distance(self):
        self.parameters['comoving_distance'] = 1
        dl = conversion.comoving_distance_to_luminosity_distance(
            self.parameters['comoving_distance'])
        self.bbh_convert()
        self.assertEqual(self.parameters['luminosity_distance'], dl)

    def test_source_to_lab_frame(self):
        self.parameters['test_source'] = 1
        self.parameters['redshift'] = 1
        lab = self.parameters['test_source'] * (1 + self.parameters['redshift'])
        self.bbh_convert()
        self.assertEqual(self.parameters['test'], lab)

    def _conversion_to_component_mass(self, keys):
        for key in keys:
            self.parameters[key] = self.mass_parameters[key]
        self.bbh_convert()
        self.assertAlmostEqual(
            max([abs(self.parameters[key] - self.component_mass_pars[key])
                 for key in ['mass_1', 'mass_2']]), 0)

    def test_chirp_mass_total_mass(self):
        self._conversion_to_component_mass(['chirp_mass', 'total_mass'])

    def test_chirp_mass_sym_mass_ratio(self):
        self._conversion_to_component_mass(['chirp_mass', 'symmetric_mass_ratio'])

    def test_chirp_mass_mass_ratio(self):
        self._conversion_to_component_mass(['chirp_mass', 'mass_ratio'])

    def test_total_mass_sym_mass_ratio(self):
        self._conversion_to_component_mass(['total_mass', 'symmetric_mass_ratio'])

    def test_total_mass_mass_ratio(self):
        self._conversion_to_component_mass(['total_mass', 'mass_ratio'])

    def test_total_mass_mass_1(self):
        self._conversion_to_component_mass(['total_mass', 'mass_1'])

    def test_total_mass_mass_2(self):
        self._conversion_to_component_mass(['total_mass', 'mass_2'])

    def test_sym_mass_ratio_mass_1(self):
        self._conversion_to_component_mass(['symmetric_mass_ratio', 'mass_1'])

    def test_sym_mass_ratio_mass_2(self):
        self._conversion_to_component_mass(['symmetric_mass_ratio', 'mass_2'])

    def test_mass_ratio_mass_1(self):
        self._conversion_to_component_mass(['mass_ratio', 'mass_1'])

    def test_mass_ratio_mass_2(self):
        self._conversion_to_component_mass(['mass_ratio', 'mass_2'])

    def test_bbh_aligned_spin_to_spherical(self):
        self.parameters['chi_1'] = -0.5
        a_1 = abs(self.parameters['chi_1'])
        tilt_1 = np.arccos(np.sign(self.parameters['chi_1']))
        phi_jl = 0.0
        phi_12 = 0.0
        self.bbh_convert()
        self.assertDictEqual(
            {key: self.parameters[key]
             for key in ['a_1', 'tilt_1', 'phi_12', 'phi_jl']},
            dict(a_1=a_1, tilt_1=tilt_1, phi_jl=phi_jl, phi_12=phi_12))

    def test_bbh_cos_angle_to_angle_conversion(self):
        self.parameters['cos_tilt_1'] = 1
        t1 = np.arccos(self.parameters['cos_tilt_1'])
        self.bbh_convert()
        self.assertEqual(self.parameters['tilt_1'], t1)

    def _conversion_to_component_tidal(self, keys):
        for key in keys:
            self.parameters[key] = self.tidal_parameters[key]
        for key in ['mass_1', 'mass_2']:
            self.parameters[key] = self.mass_parameters[key]
        self.bns_convert()
        component_dict = {
            key: self.parameters[key] for key in ['lambda_1', 'lambda_2']}
        self.assertDictEqual(component_dict, self.component_tidal_parameters)

    def test_lambda_tilde_delta_lambda_tilde(self):
        self._conversion_to_component_tidal(['lambda_tilde', 'delta_lambda_tilde'])

    def test_lambda_tilde(self):
        self._conversion_to_component_tidal(['lambda_tilde'])

    def test_lambda_1(self):
        self._conversion_to_component_tidal(['lambda_1'])


class TestGenerateAllParameters(unittest.TestCase):

    def setUp(self):
        self.parameters = dict(
            mass_1=36., mass_2=29., a_1=0.4, a_2=0.3, tilt_1=0.5, tilt_2=1.0,
            phi_12=1.7, phi_jl=0.3, luminosity_distance=2000., theta_jn=0.4,
            psi=2.659, phase=1.3, geocent_time=1126259642.413, ra=1.375,
            dec=-1.2108, lambda_tilde=1000, delta_lambda_tilde=0)
        self.expected_bbh_keys = [
            'mass_1', 'mass_2', 'a_1', 'a_2', 'tilt_1', 'tilt_2', 'phi_12',
            'phi_jl', 'luminosity_distance', 'theta_jn', 'psi', 'phase',
            'geocent_time', 'ra', 'dec', 'reference_frequency',
            'waveform_approximant', 'minimum_frequency', 'chirp_mass',
            'total_mass', 'symmetric_mass_ratio', 'mass_ratio', 'iota',
            'spin_1x', 'spin_1y', 'spin_1z', 'spin_2x', 'spin_2y', 'spin_2z',
            'phi_1', 'phi_2', 'chi_eff', 'chi_1_in_plane', 'chi_2_in_plane',
            'chi_p', 'cos_tilt_1', 'cos_tilt_2', 'redshift',
            'comoving_distance', 'mass_1_source', 'mass_2_source',
            'chirp_mass_source', 'total_mass_source'
        ]
        self.expected_tidal_keys = [
            'lambda_1', 'lambda_2', 'lambda_tilde', 'delta_lambda_tilde'
        ]

    def test_generate_all_bbh_parameters(self):
        new_parameters = bilby.gw.conversion.generate_all_bbh_parameters(
            self.parameters
        )
        for key in self.expected_bbh_keys:
            self.assertIn(key, new_parameters)

    def test_generate_all_bns_parameters(self):
        new_parameters = bilby.gw.conversion.generate_all_bns_parameters(
            self.parameters
        )
        for key in self.expected_bbh_keys + self.expected_tidal_keys:
            self.assertIn(key, new_parameters)


class TestDistanceTransformations(unittest.TestCase):

    def setUp(self):
        self.distances = np.linspace(1, 1000, 100)

    def test_luminosity_redshift_with_cosmology(self):
        z = conversion.luminosity_distance_to_redshift(self.distances, cosmology='WMAP9')
        dl = conversion.redshift_to_luminosity_distance(z, cosmology='WMAP9')
        self.assertAlmostEqual(max(abs(dl - self.distances)), 0, 4)

    def test_comoving_redshift_with_cosmology(self):
        z = conversion.comoving_distance_to_redshift(self.distances, cosmology='WMAP9')
        dc = conversion.redshift_to_comoving_distance(z, cosmology='WMAP9')
        self.assertAlmostEqual(max(abs(dc - self.distances)), 0, 4)

    def test_comoving_luminosity_with_cosmology(self):
        dc = conversion.comoving_distance_to_luminosity_distance(self.distances, cosmology='WMAP9')
        dl = conversion.luminosity_distance_to_comoving_distance(dc, cosmology='WMAP9')
        self.assertAlmostEqual(max(abs(dl - self.distances)), 0, 4)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import

import bilby
import inspect
import unittest
import mock
from mock import MagicMock
from mock import patch
import numpy as np
import scipy.signal.windows
import os
import sys
from shutil import rmtree
import logging
import deepdish as dd


class TestInterferometerGeometry(unittest.TestCase):

    def setUp(self):
        self.length = 30
        self.latitude = 1
        self.longitude = 2
        self.elevation = 3
        self.xarm_azimuth = 4
        self.yarm_azimuth = 5
        self.xarm_tilt = 0.
        self.yarm_tilt = 0.
        self.geometry = bilby.gw.detector.InterferometerGeometry(length=self.length,
                                                                 latitude=self.latitude,
                                                                 longitude=self.longitude,
                                                                 elevation=self.elevation,
                                                                 xarm_azimuth=self.xarm_azimuth,
                                                                 yarm_azimuth=self.yarm_azimuth,
                                                                 xarm_tilt=self.xarm_tilt,
                                                                 yarm_tilt=self.yarm_tilt)

    def tearDown(self):
        del self.length
        del self.latitude
        del self.longitude
        del self.elevation
        del self.xarm_azimuth
        del self.yarm_azimuth
        del self.xarm_tilt
        del self.yarm_tilt
        del self.geometry

    def test_length_setting(self):
        self.assertEqual(self.geometry.length, self.length)

    def test_latitude_setting(self):
        self.assertEqual(self.geometry.latitude, self.latitude)

    def test_longitude_setting(self):
        self.assertEqual(self.geometry.longitude, self.longitude)

    def test_elevation_setting(self):
        self.assertEqual(self.geometry.elevation, self.elevation)

    def test_xarm_azi_setting(self):
        self.assertEqual(self.geometry.xarm_azimuth, self.xarm_azimuth)

    def test_yarm_azi_setting(self):
        self.assertEqual(self.geometry.yarm_azimuth, self.yarm_azimuth)

    def test_xarm_tilt_setting(self):
        self.assertEqual(self.geometry.xarm_tilt, self.xarm_tilt)

    def test_yarm_tilt_setting(self):
        self.assertEqual(self.geometry.yarm_tilt, self.yarm_tilt)

    def test_vertex_without_update(self):
        _ = self.geometry.vertex
        with mock.patch('bilby.gw.utils.get_vertex_position_geocentric') as m:
            m.return_value = np.array([1])
            self.assertFalse(np.array_equal(self.geometry.vertex, np.array([1])))

    def test_vertex_with_latitude_update(self):
        with mock.patch('bilby.gw.utils.get_vertex_position_geocentric') as m:
            m.return_value = np.array([1])
            self.geometry.latitude = 5
            self.assertEqual(self.geometry.vertex, np.array([1]))

    def test_vertex_with_longitude_update(self):
        with mock.patch('bilby.gw.utils.get_vertex_position_geocentric') as m:
            m.return_value = np.array([1])
            self.geometry.longitude = 5
            self.assertEqual(self.geometry.vertex, np.array([1]))

    def test_vertex_with_elevation_update(self):
        with mock.patch('bilby.gw.utils.get_vertex_position_geocentric') as m:
            m.return_value = np.array([1])
            self.geometry.elevation = 5
            self.assertEqual(self.geometry.vertex, np.array([1]))

    def test_x_without_update(self):
        _ = self.geometry.x
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))

        self.assertFalse(np.array_equal(self.geometry.x,
                                        np.array([1])))

    def test_x_with_xarm_tilt_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.xarm_tilt = 0
        self.assertTrue(np.array_equal(self.geometry.x,
                                       np.array([1])))

    def test_x_with_xarm_azimuth_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.xarm_azimuth = 0
        self.assertTrue(np.array_equal(self.geometry.x,
                                       np.array([1])))

    def test_x_with_longitude_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.longitude = 0
        self.assertTrue(np.array_equal(self.geometry.x,
                                       np.array([1])))

    def test_x_with_latitude_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.latitude = 0
        self.assertTrue(np.array_equal(self.geometry.x,
                                       np.array([1])))

    def test_y_without_update(self):
        _ = self.geometry.y
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))

        self.assertFalse(np.array_equal(self.geometry.y,
                                        np.array([1])))

    def test_y_with_yarm_tilt_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.yarm_tilt = 0
        self.assertTrue(np.array_equal(self.geometry.y,
                                       np.array([1])))

    def test_y_with_yarm_azimuth_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.yarm_azimuth = 0
        self.assertTrue(np.array_equal(self.geometry.y,
                                       np.array([1])))

    def test_y_with_longitude_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.longitude = 0
        self.assertTrue(np.array_equal(self.geometry.y,
                                       np.array([1])))

    def test_y_with_latitude_update(self):
        self.geometry.unit_vector_along_arm = MagicMock(return_value=np.array([1]))
        self.geometry.latitude = 0
        self.assertTrue(np.array_equal(self.geometry.y,
                                       np.array([1])))

    def test_detector_tensor_without_update(self):
        _ = self.geometry.detector_tensor
        with mock.patch('numpy.einsum') as m:
            m.return_value = 1
            expected = np.array([[-9.24529394e-06, 1.02425803e-04, 3.24550668e-04],
                                 [1.02425803e-04, 1.37390844e-03, -8.61137566e-03],
                                 [3.24550668e-04, -8.61137566e-03, -1.36466315e-03]])
            self.assertTrue(np.allclose(expected, self.geometry.detector_tensor))

    def test_detector_tensor_with_x_azimuth_update(self):
        _ = self.geometry.detector_tensor
        with mock.patch('numpy.einsum') as m:
            m.return_value = 1
            self.geometry.xarm_azimuth = 1
            self.assertEqual(0, self.geometry.detector_tensor)

    def test_detector_tensor_with_y_azimuth_update(self):
        _ = self.geometry.detector_tensor
        with mock.patch('numpy.einsum') as m:
            m.return_value = 1
            self.geometry.yarm_azimuth = 1
            self.assertEqual(0, self.geometry.detector_tensor)

    def test_detector_tensor_with_x_tilt_update(self):
        _ = self.geometry.detector_tensor
        with mock.patch('numpy.einsum') as m:
            m.return_value = 1
            self.geometry.xarm_tilt = 1
            self.assertEqual(0, self.geometry.detector_tensor)

    def test_detector_tensor_with_y_tilt_update(self):
        _ = self.geometry.detector_tensor
        with mock.patch('numpy.einsum') as m:
            m.return_value = 1
            self.geometry.yarm_tilt = 1
            self.assertEqual(0, self.geometry.detector_tensor)

    def test_detector_tensor_with_longitude_update(self):
        with mock.patch('numpy.einsum') as m:
            m.return_value = 1
            self.geometry.longitude = 1
            self.assertEqual(0, self.geometry.detector_tensor)

    def test_detector_tensor_with_latitude_update(self):
        with mock.patch('numpy.einsum') as m:
            _ = self.geometry.detector_tensor
            m.return_value = 1
            self.geometry.latitude = 1
            self.assertEqual(self.geometry.detector_tensor, 0)

    def test_unit_vector_along_arm_default(self):
        with self.assertRaises(ValueError):
            self.geometry.unit_vector_along_arm('z')

    def test_unit_vector_along_arm_x(self):
        with mock.patch('numpy.array') as m:
            m.return_value = 1
            self.geometry.xarm_tilt = 0
            self.geometry.xarm_azimuth = 0
            self.geometry.yarm_tilt = 0
            self.geometry.yarm_azimuth = 90
            self.assertAlmostEqual(self.geometry.unit_vector_along_arm('x'), 1)

    def test_unit_vector_along_arm_y(self):
        with mock.patch('numpy.array') as m:
            m.return_value = 1
            self.geometry.xarm_tilt = 0
            self.geometry.xarm_azimuth = 90
            self.geometry.yarm_tilt = 0
            self.geometry.yarm_azimuth = 180
            self.assertAlmostEqual(self.geometry.unit_vector_along_arm('y'), -1)

    def test_repr(self):
        expected = 'InterferometerGeometry(length={}, latitude={}, longitude={}, elevation={}, xarm_azimuth={}, ' \
                   'yarm_azimuth={}, xarm_tilt={}, yarm_tilt={})' \
            .format(float(self.length), float(self.latitude), float(self.longitude), float(self.elevation),
                    float(self.xarm_azimuth), float(self.yarm_azimuth), float(self.xarm_tilt), float(self.yarm_tilt))
        self.assertEqual(expected, repr(self.geometry))


class TestInterferometer(unittest.TestCase):

    def setUp(self):
        self.name = 'name'
        self.power_spectral_density = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        self.length = 30
        self.latitude = 1
        self.longitude = 2
        self.elevation = 3
        self.xarm_azimuth = 4
        self.yarm_azimuth = 5
        self.xarm_tilt = 0.
        self.yarm_tilt = 0.
        # noinspection PyTypeChecker
        self.ifo = bilby.gw.detector.Interferometer(name=self.name, power_spectral_density=self.power_spectral_density,
                                                    minimum_frequency=self.minimum_frequency,
                                                    maximum_frequency=self.maximum_frequency, length=self.length,
                                                    latitude=self.latitude, longitude=self.longitude,
                                                    elevation=self.elevation,
                                                    xarm_azimuth=self.xarm_azimuth, yarm_azimuth=self.yarm_azimuth,
                                                    xarm_tilt=self.xarm_tilt, yarm_tilt=self.yarm_tilt)
        self.ifo.strain_data.set_from_frequency_domain_strain(
            np.linspace(0, 4096, 4097), sampling_frequency=4096, duration=2)
        self.outdir = 'outdir'

        self.injection_polarizations = dict()
        np.random.seed(42)
        self.injection_polarizations['plus'] = np.random.random(4097)
        self.injection_polarizations['cross'] = np.random.random(4097)

        self.waveform_generator = MagicMock()
        self.wg_polarizations = dict(plus=np.random.random(4097), cross=np.random.random(4097))
        self.waveform_generator.frequency_domain_strain = lambda _: self.wg_polarizations
        self.parameters = dict(ra=0., dec=0., geocent_time=0., psi=0.)

        bilby.core.utils.check_directory_exists_and_if_not_mkdir(self.outdir)

    def tearDown(self):
        del self.name
        del self.power_spectral_density
        del self.minimum_frequency
        del self.maximum_frequency
        del self.length
        del self.latitude
        del self.longitude
        del self.elevation
        del self.xarm_azimuth
        del self.yarm_azimuth
        del self.xarm_tilt
        del self.yarm_tilt
        del self.ifo
        del self.injection_polarizations
        del self.wg_polarizations
        del self.waveform_generator
        del self.parameters
        rmtree(self.outdir)

    def test_name_setting(self):
        self.assertEqual(self.ifo.name, self.name)

    def test_psd_setting(self):
        self.assertEqual(self.ifo.power_spectral_density, self.power_spectral_density)

    def test_min_freq_setting(self):
        self.assertEqual(self.ifo.strain_data.minimum_frequency, self.minimum_frequency)

    def test_max_freq_setting(self):
        self.assertEqual(self.ifo.strain_data.maximum_frequency, self.maximum_frequency)

    def test_antenna_response_default(self):
        with mock.patch('bilby.gw.utils.get_polarization_tensor') as m:
            with mock.patch('numpy.einsum') as n:
                m.return_value = 0
                n.return_value = 1
                self.assertEqual(self.ifo.antenna_response(234, 52, 54, 76, 'plus'), 1)

    def test_antenna_response_einsum(self):
        with mock.patch('bilby.gw.utils.get_polarization_tensor') as m:
            m.return_value = np.ones((3, 3))
            self.assertAlmostEqual(self.ifo.antenna_response(234, 52, 54, 76, 'plus'), self.ifo.detector_tensor.sum())

    def test_get_detector_response_default_behaviour(self):
        self.ifo.antenna_response = MagicMock(return_value=1)
        self.ifo.time_delay_from_geocenter = MagicMock(return_value=0)
        self.ifo.epoch = 0
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        # self.ifo.frequency_array = np.array([8, 12, 16, 20, 24])
        plus = np.linspace(0, 4096, 4097)
        response = self.ifo.get_detector_response(
            waveform_polarizations=dict(plus=plus),
            parameters=dict(ra=0, dec=0, geocent_time=0, psi=0))
        self.assertTrue(np.array_equal(response, plus * self.ifo.frequency_mask * np.exp(-0j)))

    def test_get_detector_response_with_dt(self):
        self.ifo.antenna_response = MagicMock(return_value=1)
        self.ifo.time_delay_from_geocenter = MagicMock(return_value=0)
        self.ifo.epoch = 1
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        plus = np.linspace(0, 4096, 4097)
        response = self.ifo.get_detector_response(
            waveform_polarizations=dict(plus=plus),
            parameters=dict(ra=0, dec=0, geocent_time=0, psi=0))
        expected_response = plus * self.ifo.frequency_mask * np.exp(-1j * 2 * np.pi * self.ifo.frequency_array)
        self.assertTrue(np.allclose(abs(expected_response), abs(response)))

    def test_get_detector_response_multiple_modes(self):
        self.ifo.antenna_response = MagicMock(return_value=1)
        self.ifo.time_delay_from_geocenter = MagicMock(return_value=0)
        self.ifo.epoch = 0
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        plus = np.linspace(0, 4096, 4097)
        cross = np.linspace(0, 4096, 4097)
        response = self.ifo.get_detector_response(
            waveform_polarizations=dict(plus=plus, cross=cross),
            parameters=dict(ra=0, dec=0, geocent_time=0, psi=0))
        self.assertTrue(np.array_equal(response, (plus + cross) * self.ifo.frequency_mask * np.exp(-0j)))

    def test_inject_signal_from_waveform_polarizations_correct_injection(self):
        original_strain = self.ifo.strain_data.frequency_domain_strain
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        self.ifo.inject_signal_from_waveform_polarizations(parameters=self.parameters,
                                                           injection_polarizations=self.injection_polarizations)
        expected = self.injection_polarizations['plus'] + self.injection_polarizations['cross'] + original_strain
        self.assertTrue(np.array_equal(expected, self.ifo.strain_data._frequency_domain_strain))

    def test_inject_signal_from_waveform_polarizations_update_time_domain_strain(self):
        original_td_strain = self.ifo.strain_data.time_domain_strain
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        self.ifo.inject_signal_from_waveform_polarizations(parameters=self.parameters,
                                                           injection_polarizations=self.injection_polarizations)
        self.assertFalse(np.array_equal(original_td_strain, self.ifo.strain_data.time_domain_strain))

    def test_inject_signal_from_waveform_polarizations_meta_data(self):
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        self.ifo.inject_signal_from_waveform_polarizations(parameters=self.parameters,
                                                           injection_polarizations=self.injection_polarizations)
        signal_ifo_expected = self.injection_polarizations['plus'] + self.injection_polarizations['cross']
        self.assertAlmostEqual(self.ifo.optimal_snr_squared(signal=signal_ifo_expected).real,
                               self.ifo.meta_data['optimal_SNR']**2, 10)
        self.assertAlmostEqual(self.ifo.matched_filter_snr(signal=signal_ifo_expected),
                               self.ifo.meta_data['matched_filter_SNR'], 10)
        self.assertDictEqual(self.parameters,
                             self.ifo.meta_data['parameters'])

    def test_inject_signal_from_waveform_polarizations_incorrect_length(self):
        self.injection_polarizations['plus'] = np.random.random(1000)
        self.injection_polarizations['cross'] = np.random.random(1000)
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        with self.assertRaises(ValueError):
            self.ifo.inject_signal_from_waveform_polarizations(parameters=self.parameters,
                                                               injection_polarizations=self.injection_polarizations)

    @patch.object(bilby.core.utils.logger, 'warning')
    def test_inject_signal_outside_segment_logs_warning(self, m):
        self.parameters['geocent_time'] = 24345.
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        self.ifo.inject_signal_from_waveform_polarizations(parameters=self.parameters,
                                                           injection_polarizations=self.injection_polarizations)
        self.assertTrue(m.called)

    def test_inject_signal_from_waveform_generator_correct_return_value(self):
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        returned_polarizations = self.ifo.inject_signal_from_waveform_generator(parameters=self.parameters,
                                                                                waveform_generator=self.waveform_generator)
        self.assertTrue(np.array_equal(self.wg_polarizations['plus'], returned_polarizations['plus']))
        self.assertTrue(np.array_equal(self.wg_polarizations['cross'], returned_polarizations['cross']))

    @patch.object(bilby.gw.detector.Interferometer, 'inject_signal_from_waveform_generator')
    def test_inject_signal_with_waveform_generator_correct_call(self, m):
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        _ = self.ifo.inject_signal(parameters=self.parameters,
                                   waveform_generator=self.waveform_generator)
        m.assert_called_with(parameters=self.parameters,
                             waveform_generator=self.waveform_generator)

    def test_inject_signal_from_waveform_generator_correct_injection(self):
        original_strain = self.ifo.strain_data.frequency_domain_strain
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        injection_polarizations = self.ifo.inject_signal_from_waveform_generator(parameters=self.parameters,
                                                                                 waveform_generator=self.waveform_generator)
        expected = injection_polarizations['plus'] + injection_polarizations['cross'] + original_strain
        self.assertTrue(np.array_equal(expected, self.ifo.strain_data._frequency_domain_strain))

    def test_inject_signal_with_injection_polarizations(self):
        original_strain = self.ifo.strain_data.frequency_domain_strain
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        self.ifo.inject_signal(parameters=self.parameters,
                               injection_polarizations=self.injection_polarizations)
        expected = self.injection_polarizations['plus'] + self.injection_polarizations['cross'] + original_strain
        self.assertTrue(np.array_equal(expected, self.ifo.strain_data._frequency_domain_strain))

    @patch.object(bilby.gw.detector.Interferometer, 'inject_signal_from_waveform_polarizations')
    def test_inject_signal_with_injection_polarizations_and_waveform_generator(self, m):
        self.ifo.get_detector_response = lambda x, params: x['plus'] + x['cross']
        _ = self.ifo.inject_signal(parameters=self.parameters,
                                   waveform_generator=self.waveform_generator,
                                   injection_polarizations=self.injection_polarizations)
        m.assert_called_with(parameters=self.parameters,
                             injection_polarizations=self.injection_polarizations)
        with self.assertRaises(ValueError):
            m.assert_called_with(parameters=self.parameters,
                                 injection_polarizations=self.wg_polarizations)

    def test_inject_signal_raises_value_error(self):
        with self.assertRaises(ValueError):
            self.ifo.inject_signal(injection_polarizations=None, parameters=None)

    def test_time_delay_from_geocenter(self):
        with mock.patch('bilby.gw.utils.time_delay_geocentric') as m:
            m.return_value = 1
            self.assertEqual(self.ifo.time_delay_from_geocenter(1, 2, 3), 1)

    def test_vertex_position_geocentric(self):
        with mock.patch('bilby.gw.utils.get_vertex_position_geocentric') as m:
            m.return_value = 1
            self.assertEqual(self.ifo.vertex_position_geocentric(), 1)

    def test_optimal_snr_squared(self):
        """
        Merely checks parameters are given in the right order and the frequency
        mask is applied.
        """
        with mock.patch('bilby.gw.utils.noise_weighted_inner_product') as m:
            m.side_effect = lambda a, b, c, d: [a, b, c, d]
            signal = np.ones_like(self.ifo.power_spectral_density_array)
            mask = self.ifo.frequency_mask
            expected = [signal[mask], signal[mask],
                        self.ifo.power_spectral_density_array[mask],
                        self.ifo.strain_data.duration]
            actual = self.ifo.optimal_snr_squared(signal=signal)
            self.assertTrue(np.array_equal(expected[0], actual[0]))
            self.assertTrue(np.array_equal(expected[1], actual[1]))
            self.assertTrue(np.array_equal(expected[2], actual[2]))
            self.assertEqual(expected[3], actual[3])

    def test_repr(self):
        expected = 'Interferometer(name=\'{}\', power_spectral_density={}, minimum_frequency={}, ' \
                   'maximum_frequency={}, length={}, latitude={}, longitude={}, elevation={}, xarm_azimuth={}, ' \
                   'yarm_azimuth={}, xarm_tilt={}, yarm_tilt={})' \
            .format(self.name, self.power_spectral_density, float(self.minimum_frequency),
                    float(self.maximum_frequency), float(self.length), float(self.latitude), float(self.longitude),
                    float(self.elevation), float(self.xarm_azimuth), float(self.yarm_azimuth), float(self.xarm_tilt),
                    float(self.yarm_tilt))
        self.assertEqual(expected, repr(self.ifo))

    def test_to_and_from_hdf5_loading(self):
        if sys.version_info[0] < 3:
            with self.assertRaises(NotImplementedError):
                self.ifo.to_hdf5(outdir='outdir', label='test')
        else:
            self.ifo.to_hdf5(outdir='outdir', label='test')
            filename = self.ifo._hdf5_filename_from_outdir_label(outdir='outdir', label='test')
            recovered_ifo = bilby.gw.detector.Interferometer.from_hdf5(filename)
            self.assertEqual(self.ifo, recovered_ifo)

    def test_to_and_from_hdf5_wrong_class(self):
        if sys.version_info[0] < 3:
            pass
        else:
            bilby.core.utils.check_directory_exists_and_if_not_mkdir('outdir')
            dd.io.save('./outdir/psd.h5', self.power_spectral_density)
            filename = self.ifo._hdf5_filename_from_outdir_label(outdir='outdir', label='psd')
            with self.assertRaises(TypeError):
                bilby.gw.detector.Interferometer.from_hdf5(filename)


class TestInterferometerEquals(unittest.TestCase):

    def setUp(self):
        self.name = 'name'
        self.power_spectral_density_1 = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.power_spectral_density_2 = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        self.length = 30
        self.latitude = 1
        self.longitude = 2
        self.elevation = 3
        self.xarm_azimuth = 4
        self.yarm_azimuth = 5
        self.xarm_tilt = 0.
        self.yarm_tilt = 0.
        # noinspection PyTypeChecker
        self.duration = 1
        self.sampling_frequency = 200
        self.frequency_array = bilby.utils.create_frequency_series(sampling_frequency=self.sampling_frequency,
                                                                   duration=self.duration)
        self.strain = self.frequency_array
        self.ifo_1 = bilby.gw.detector.Interferometer(name=self.name,
                                                      power_spectral_density=self.power_spectral_density_1,
                                                      minimum_frequency=self.minimum_frequency,
                                                      maximum_frequency=self.maximum_frequency, length=self.length,
                                                      latitude=self.latitude, longitude=self.longitude,
                                                      elevation=self.elevation,
                                                      xarm_azimuth=self.xarm_azimuth, yarm_azimuth=self.yarm_azimuth,
                                                      xarm_tilt=self.xarm_tilt, yarm_tilt=self.yarm_tilt)
        self.ifo_2 = bilby.gw.detector.Interferometer(name=self.name,
                                                      power_spectral_density=self.power_spectral_density_2,
                                                      minimum_frequency=self.minimum_frequency,
                                                      maximum_frequency=self.maximum_frequency, length=self.length,
                                                      latitude=self.latitude, longitude=self.longitude,
                                                      elevation=self.elevation,
                                                      xarm_azimuth=self.xarm_azimuth, yarm_azimuth=self.yarm_azimuth,
                                                      xarm_tilt=self.xarm_tilt, yarm_tilt=self.yarm_tilt)
        self.ifo_1.set_strain_data_from_frequency_domain_strain(frequency_array=self.frequency_array,
                                                                frequency_domain_strain=self.strain)
        self.ifo_2.set_strain_data_from_frequency_domain_strain(frequency_array=self.frequency_array,
                                                                frequency_domain_strain=self.strain)

    def tearDown(self):
        del self.name
        del self.power_spectral_density_1
        del self.power_spectral_density_2
        del self.minimum_frequency
        del self.maximum_frequency
        del self.length
        del self.latitude
        del self.longitude
        del self.elevation
        del self.xarm_azimuth
        del self.yarm_azimuth
        del self.xarm_tilt
        del self.yarm_tilt
        del self.ifo_1
        del self.ifo_2
        del self.sampling_frequency
        del self.duration
        del self.frequency_array
        del self.strain

    def test_eq_true(self):
        self.assertEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_psd(self):
        self.ifo_1.power_spectral_density.psd_array[0] = 1234
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_minimum_frequency(self):
        self.ifo_1.minimum_frequency -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_maximum_frequency(self):
        self.ifo_1.minimum_frequency -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_length(self):
        self.ifo_1.length -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_latitude(self):
        self.ifo_1.latitude -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_longitude(self):
        self.ifo_1.longitude -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_elevation(self):
        self.ifo_1.elevation -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_xarm_azimuth(self):
        self.ifo_1.xarm_azimuth -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_xarmtilt(self):
        self.ifo_1.xarm_tilt -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_yarm_azimuth(self):
        self.ifo_1.yarm_azimuth -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_yarm_tilt(self):
        self.ifo_1.yarm_tilt -= 1
        self.assertNotEqual(self.ifo_1, self.ifo_2)

    def test_eq_false_different_ifo_strain_data(self):
        self.strain = bilby.utils.create_frequency_series(sampling_frequency=self.sampling_frequency / 2,
                                                          duration=self.duration * 2)
        self.ifo_1.set_strain_data_from_frequency_domain_strain(frequency_array=self.frequency_array,
                                                                frequency_domain_strain=self.strain)
        self.assertNotEqual(self.ifo_1, self.ifo_2)


class TestInterferometerStrainData(unittest.TestCase):

    def setUp(self):
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        self.ifosd = bilby.gw.detector.InterferometerStrainData(
            minimum_frequency=self.minimum_frequency,
            maximum_frequency=self.maximum_frequency)

    def tearDown(self):
        del self.minimum_frequency
        del self.maximum_frequency
        del self.ifosd

    def test_frequency_mask(self):
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        with mock.patch('bilby.core.utils.create_frequency_series') as m:
            m.return_value = np.array([5, 15, 25])
            self.ifosd.set_from_frequency_domain_strain(
                frequency_domain_strain=np.array([0, 1, 2]), frequency_array=np.array([5, 15, 25]))
            self.assertTrue(np.array_equal(self.ifosd.frequency_mask, [False, True, False]))

    def test_set_data_fails(self):
        with mock.patch('bilby.core.utils.create_frequency_series') as m:
            m.return_value = [1, 2, 3]
            with self.assertRaises(ValueError):
                self.ifosd.set_from_frequency_domain_strain(
                    frequency_domain_strain=np.array([0, 1, 2]))

    def test_set_data_fails_too_much(self):
        with mock.patch('bilby.core.utils.create_frequency_series') as m:
            m.return_value = [1, 2, 3]
            with self.assertRaises(ValueError):
                self.ifosd.set_from_frequency_domain_strain(
                    frequency_domain_strain=np.array([0, 1, 2]), frequency_array=np.array([1, 2, 3]),
                    duration=3, sampling_frequency=1)

    def test_start_time_init(self):
        with mock.patch('bilby.core.utils.create_frequency_series') as m:
            m.return_value = [1, 2, 3]
            duration = 3
            sampling_frequency = 1
            self.ifosd.set_from_frequency_domain_strain(
                frequency_domain_strain=np.array([0, 1, 2]), duration=duration,
                sampling_frequency=sampling_frequency)
            self.assertTrue(self.ifosd.start_time == 0)

    def test_start_time_set(self):
        with mock.patch('bilby.core.utils.create_frequency_series') as m:
            m.return_value = [1, 2, 3]
            duration = 3
            sampling_frequency = 1
            self.ifosd.set_from_frequency_domain_strain(
                frequency_domain_strain=np.array([0, 1, 2]), duration=duration,
                sampling_frequency=sampling_frequency, start_time=10)
            self.assertTrue(self.ifosd.start_time == 10)

    def test_time_array_frequency_array_consistency(self):
        duration = 1
        sampling_frequency = 10
        time_array = bilby.core.utils.create_time_series(
            sampling_frequency=sampling_frequency, duration=duration)
        time_domain_strain = np.random.normal(
            0, duration - 1 / sampling_frequency, len(time_array))
        self.ifosd.roll_off = 0
        self.ifosd.set_from_time_domain_strain(
            time_domain_strain=time_domain_strain, duration=duration,
            sampling_frequency=sampling_frequency)

        frequency_domain_strain, freqs = bilby.core.utils.nfft(
            time_domain_strain, sampling_frequency)

        self.assertTrue(np.all(
            self.ifosd.frequency_domain_strain == frequency_domain_strain * self.ifosd.frequency_mask))

    def test_time_within_data_before(self):
        self.ifosd.start_time = 3
        self.ifosd.duration = 2
        self.assertFalse(self.ifosd.time_within_data(2))

    def test_time_within_data_during(self):
        self.ifosd.start_time = 3
        self.ifosd.duration = 2
        self.assertTrue(self.ifosd.time_within_data(3))
        self.assertTrue(self.ifosd.time_within_data(4))
        self.assertTrue(self.ifosd.time_within_data(5))

    def test_time_within_data_after(self):
        self.ifosd.start_time = 3
        self.ifosd.duration = 2
        self.assertFalse(self.ifosd.time_within_data(6))

    def test_time_domain_window_no_roll_off_no_alpha(self):
        self.ifosd._time_domain_strain = np.array([3])
        self.ifosd.duration = 5
        self.ifosd.roll_off = 2
        expected_window = scipy.signal.windows.tukey(len(self.ifosd._time_domain_strain), alpha=self.ifosd.alpha)
        self.assertEqual(expected_window,
                         self.ifosd.time_domain_window())
        self.assertEqual(np.mean(expected_window ** 2), self.ifosd.window_factor)

    def test_time_domain_window_sets_roll_off_directly(self):
        self.ifosd._time_domain_strain = np.array([3])
        self.ifosd.duration = 5
        self.ifosd.roll_off = 2
        expected_roll_off = 6
        self.ifosd.time_domain_window(roll_off=expected_roll_off)
        self.assertEqual(expected_roll_off, self.ifosd.roll_off)

    def test_time_domain_window_sets_roll_off_indirectly(self):
        self.ifosd._time_domain_strain = np.array([3])
        self.ifosd.duration = 5
        self.ifosd.roll_off = 2
        alpha = 4
        expected_roll_off = alpha * self.ifosd.duration / 2
        self.ifosd.time_domain_window(alpha=alpha)
        self.assertEqual(expected_roll_off, self.ifosd.roll_off)

    def test_time_domain_strain_when_set(self):
        expected_strain = 5
        self.ifosd._time_domain_strain = expected_strain
        self.assertEqual(expected_strain, self.ifosd.time_domain_strain)

    @patch('bilby.core.utils.infft')
    def test_time_domain_strain_from_frequency_domain_strain(self, m):
        m.return_value = 5
        self.ifosd.sampling_frequency = 200
        self.ifosd.duration = 4
        self.ifosd.sampling_frequency = 123
        self.ifosd.frequency_domain_strain = self.ifosd.frequency_array
        self.assertEqual(m.return_value, self.ifosd.time_domain_strain)

    def test_time_domain_strain_not_set(self):
        self.ifosd._time_domain_strain = None
        self.ifosd._frequency_domain_strain = None
        with self.assertRaises(ValueError):
            test = self.ifosd.time_domain_strain

    def test_frequency_domain_strain_when_set(self):
        self.ifosd.sampling_frequency = 200
        self.ifosd.duration = 4
        expected_strain = self.ifosd.frequency_array * self.ifosd.frequency_mask
        self.ifosd._frequency_domain_strain = expected_strain
        self.assertTrue(np.array_equal(expected_strain,
                                       self.ifosd.frequency_domain_strain))

    @patch('bilby.core.utils.nfft')
    def test_frequency_domain_strain_from_frequency_domain_strain(self, m):
        self.ifosd.start_time = 0
        self.ifosd.duration = 4
        self.ifosd.sampling_frequency = 200
        m.return_value = self.ifosd.frequency_array, self.ifosd.frequency_array
        self.ifosd._time_domain_strain = self.ifosd.time_array
        self.assertTrue(np.array_equal(self.ifosd.frequency_array * self.ifosd.frequency_mask,
                                       self.ifosd.frequency_domain_strain))

    def test_frequency_domain_strain_not_set(self):
        self.ifosd._time_domain_strain = None
        self.ifosd._frequency_domain_strain = None
        with self.assertRaises(ValueError):
            test = self.ifosd.frequency_domain_strain

    def test_set_frequency_domain_strain(self):
        self.ifosd.duration = 4
        self.ifosd.sampling_frequency = 200
        self.ifosd.frequency_domain_strain = np.ones(len(self.ifosd.frequency_array))
        self.assertTrue(np.array_equal(np.ones(len(self.ifosd.frequency_array)),
                                       self.ifosd._frequency_domain_strain))

    def test_set_frequency_domain_strain_wrong_length(self):
        self.ifosd.duration = 4
        self.ifosd.sampling_frequency = 200
        with self.assertRaises(ValueError):
            self.ifosd.frequency_domain_strain = np.array([1])


class TestInterferometerStrainDataEquals(unittest.TestCase):

    def setUp(self):
        self.minimum_frequency = 10
        self.maximum_frequency = 20
        self.roll_off = 0.2
        self.sampling_frequency = 100
        self.duration = 2
        self.frequency_array = bilby.utils.create_frequency_series(sampling_frequency=self.sampling_frequency,
                                                                   duration=self.duration)
        self.strain = self.frequency_array
        self.ifosd_1 = bilby.gw.detector.InterferometerStrainData(minimum_frequency=self.minimum_frequency,
                                                                  maximum_frequency=self.maximum_frequency,
                                                                  roll_off=self.roll_off)
        self.ifosd_2 = bilby.gw.detector.InterferometerStrainData(minimum_frequency=self.minimum_frequency,
                                                                  maximum_frequency=self.maximum_frequency,
                                                                  roll_off=self.roll_off)
        self.ifosd_1.set_from_frequency_domain_strain(frequency_domain_strain=self.strain,
                                                      frequency_array=self.frequency_array)
        self.ifosd_2.set_from_frequency_domain_strain(frequency_domain_strain=self.strain,
                                                      frequency_array=self.frequency_array)

    def tearDown(self):
        del self.minimum_frequency
        del self.maximum_frequency
        del self.roll_off
        del self.sampling_frequency
        del self.duration
        del self.frequency_array
        del self.strain
        del self.ifosd_1
        del self.ifosd_2

    def test_eq_true(self):
        self.assertEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_minimum_frequency(self):
        self.ifosd_1.minimum_frequency -= 1
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_maximum_frequency(self):
        self.ifosd_1.maximum_frequency -= 1
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_roll_off(self):
        self.ifosd_1.roll_off -= 0.1
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_window_factor(self):
        self.ifosd_1.roll_off -= 0.1
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_sampling_frequency(self):
        self.ifosd_1.sampling_frequency *= 2
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_sampling_duration(self):
        self.ifosd_1.duration *= 2
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_start_time(self):
        self.ifosd_1.start_time -= 0.1
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_frequency_array(self):
        new_frequency_array = bilby.utils.create_frequency_series(sampling_frequency=self.sampling_frequency / 2,
                                                                  duration=self.duration * 2)
        self.ifosd_1.frequency_array = new_frequency_array
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_frequency_domain_strain(self):
        new_strain = bilby.utils.create_frequency_series(sampling_frequency=self.sampling_frequency / 2,
                                                         duration=self.duration * 2)
        self.ifosd_1._frequency_domain_strain = new_strain
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_time_array(self):
        new_time_array = bilby.utils.create_time_series(sampling_frequency=self.sampling_frequency / 2,
                                                        duration=self.duration * 2)
        self.ifosd_1.time_array = new_time_array
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)

    def test_eq_different_time_domain_strain(self):
        new_strain = bilby.utils.create_time_series(sampling_frequency=self.sampling_frequency / 2,
                                                    duration=self.duration * 2)
        self.ifosd_1._time_domain_strain = new_strain
        self.assertNotEqual(self.ifosd_1, self.ifosd_2)


class TestInterferometerList(unittest.TestCase):

    def setUp(self):
        self.frequency_arrays = np.linspace(0, 4096, 4097)
        self.name1 = 'name1'
        self.name2 = 'name2'
        self.power_spectral_density1 = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.power_spectral_density2 = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.minimum_frequency1 = 10
        self.minimum_frequency2 = 10
        self.maximum_frequency1 = 20
        self.maximum_frequency2 = 20
        self.length1 = 30
        self.length2 = 30
        self.latitude1 = 1
        self.latitude2 = 1
        self.longitude1 = 2
        self.longitude2 = 2
        self.elevation1 = 3
        self.elevation2 = 3
        self.xarm_azimuth1 = 4
        self.xarm_azimuth2 = 4
        self.yarm_azimuth1 = 5
        self.yarm_azimuth2 = 5
        self.xarm_tilt1 = 0.
        self.xarm_tilt2 = 0.
        self.yarm_tilt1 = 0.
        self.yarm_tilt2 = 0.
        # noinspection PyTypeChecker
        self.ifo1 = bilby.gw.detector.Interferometer(name=self.name1,
                                                     power_spectral_density=self.power_spectral_density1,
                                                     minimum_frequency=self.minimum_frequency1,
                                                     maximum_frequency=self.maximum_frequency1, length=self.length1,
                                                     latitude=self.latitude1, longitude=self.longitude1,
                                                     elevation=self.elevation1,
                                                     xarm_azimuth=self.xarm_azimuth1, yarm_azimuth=self.yarm_azimuth1,
                                                     xarm_tilt=self.xarm_tilt1, yarm_tilt=self.yarm_tilt1)
        self.ifo2 = bilby.gw.detector.Interferometer(name=self.name2,
                                                     power_spectral_density=self.power_spectral_density2,
                                                     minimum_frequency=self.minimum_frequency2,
                                                     maximum_frequency=self.maximum_frequency2, length=self.length2,
                                                     latitude=self.latitude2, longitude=self.longitude2,
                                                     elevation=self.elevation2,
                                                     xarm_azimuth=self.xarm_azimuth2, yarm_azimuth=self.yarm_azimuth2,
                                                     xarm_tilt=self.xarm_tilt2, yarm_tilt=self.yarm_tilt2)
        self.ifo1.strain_data.set_from_frequency_domain_strain(
            self.frequency_arrays, sampling_frequency=4096, duration=2)
        self.ifo2.strain_data.set_from_frequency_domain_strain(
            self.frequency_arrays, sampling_frequency=4096, duration=2)
        self.ifo_list = bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])
        self.outdir = 'outdir'
        bilby.core.utils.check_directory_exists_and_if_not_mkdir(self.outdir)

    def tearDown(self):
        del self.frequency_arrays
        del self.name1
        del self.name2
        del self.power_spectral_density1
        del self.power_spectral_density2
        del self.minimum_frequency1
        del self.minimum_frequency2
        del self.maximum_frequency1
        del self.maximum_frequency2
        del self.length1
        del self.length2
        del self.latitude1
        del self.latitude2
        del self.longitude1
        del self.longitude2
        del self.elevation1
        del self.elevation2
        del self.xarm_azimuth1
        del self.xarm_azimuth2
        del self.yarm_azimuth1
        del self.yarm_azimuth2
        del self.xarm_tilt1
        del self.xarm_tilt2
        del self.yarm_tilt1
        del self.yarm_tilt2
        del self.ifo1
        del self.ifo2
        del self.ifo_list
        rmtree(self.outdir)

    def test_init_with_string(self):
        with self.assertRaises(TypeError):
            bilby.gw.detector.InterferometerList("string")

    def test_init_with_string_list(self):
        """ Merely checks if this ends up in the right bracket """
        with mock.patch('bilby.gw.detector.networks.get_empty_interferometer') as m:
            m.side_effect = TypeError
            with self.assertRaises(TypeError):
                bilby.gw.detector.InterferometerList(['string'])

    def test_init_with_other_object(self):
        with self.assertRaises(TypeError):
            bilby.gw.detector.InterferometerList([object()])

    def test_init_with_actual_ifos(self):
        ifo_list = bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])
        self.assertEqual(self.ifo1, ifo_list[0])
        self.assertEqual(self.ifo2, ifo_list[1])

    def test_init_inconsistent_duration(self):
        self.frequency_arrays = np.linspace(0, 2048, 2049)
        self.ifo2 = bilby.gw.detector.Interferometer(name=self.name2,
                                                     power_spectral_density=self.power_spectral_density2,
                                                     minimum_frequency=self.minimum_frequency2,
                                                     maximum_frequency=self.maximum_frequency2, length=self.length2,
                                                     latitude=self.latitude2, longitude=self.longitude2,
                                                     elevation=self.elevation2,
                                                     xarm_azimuth=self.xarm_azimuth2, yarm_azimuth=self.yarm_azimuth2,
                                                     xarm_tilt=self.xarm_tilt2, yarm_tilt=self.yarm_tilt2)
        self.ifo2.strain_data.set_from_frequency_domain_strain(
            self.frequency_arrays, sampling_frequency=4096, duration=1)
        with self.assertRaises(ValueError):
            bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])

    def test_init_inconsistent_sampling_frequency(self):
        self.frequency_arrays = np.linspace(0, 2048, 2049)
        self.ifo2 = bilby.gw.detector.Interferometer(name=self.name2,
                                                     power_spectral_density=self.power_spectral_density2,
                                                     minimum_frequency=self.minimum_frequency2,
                                                     maximum_frequency=self.maximum_frequency2, length=self.length2,
                                                     latitude=self.latitude2, longitude=self.longitude2,
                                                     elevation=self.elevation2,
                                                     xarm_azimuth=self.xarm_azimuth2, yarm_azimuth=self.yarm_azimuth2,
                                                     xarm_tilt=self.xarm_tilt2, yarm_tilt=self.yarm_tilt2)
        self.ifo2.strain_data.set_from_frequency_domain_strain(
            self.frequency_arrays, sampling_frequency=2048, duration=2)
        with self.assertRaises(ValueError):
            bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])

    def test_init_inconsistent_start_time(self):
        self.ifo2.strain_data.start_time = 1
        with self.assertRaises(ValueError):
            bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])

    @patch.object(bilby.gw.detector.networks.logger, 'warning')
    def test_check_interferometers_relative_tolerance(self, mock_warning):
        # Value larger than relative tolerance -- not tolerated
        self.ifo2.strain_data.start_time = self.ifo1.strain_data.start_time + 1e-4
        with self.assertRaises(ValueError):
            bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])

        # Value smaller than relative tolerance -- tolerated with warning
        self.ifo2.strain_data.start_time = self.ifo1.strain_data.start_time + 1e-6
        ifo_list = bilby.gw.detector.InterferometerList([self.ifo1, self.ifo2])
        self.assertIsNotNone(ifo_list)
        self.assertTrue(mock_warning.called)
        warning_log_str = mock_warning.call_args.args[0].args[0]
        self.assertIsInstance(warning_log_str, str)
        self.assertTrue("The start_time of all interferometers are not the same:" in
                        warning_log_str)

    @patch.object(bilby.gw.detector.Interferometer, 'set_strain_data_from_power_spectral_density')
    def test_set_strain_data_from_power_spectral_density(self, m):
        self.ifo_list.set_strain_data_from_power_spectral_densities(sampling_frequency=123, duration=6.2, start_time=3)
        m.assert_called_with(sampling_frequency=123, duration=6.2, start_time=3)
        self.assertEqual(len(self.ifo_list), m.call_count)

    def test_inject_signal_pol_and_wg_none(self):
        with self.assertRaises(ValueError):
            self.ifo_list.inject_signal(injection_polarizations=None, waveform_generator=None)

    def test_meta_data(self):
        ifos_list = [self.ifo1, self.ifo2]
        ifos = bilby.gw.detector.InterferometerList(ifos_list)
        self.assertTrue(isinstance(ifos.meta_data, dict))
        meta_data = {ifo.name: ifo.meta_data for ifo in ifos_list}
        self.assertEqual(ifos.meta_data, meta_data)

    @patch.object(bilby.gw.waveform_generator.WaveformGenerator, 'frequency_domain_strain')
    def test_inject_signal_pol_none_calls_frequency_domain_strain(self, m):
        waveform_generator = bilby.gw.waveform_generator.WaveformGenerator(
            frequency_domain_source_model=lambda x, y, z: x)
        self.ifo1.inject_signal = MagicMock(return_value=None)
        self.ifo2.inject_signal = MagicMock(return_value=None)
        self.ifo_list.inject_signal(parameters=None, waveform_generator=waveform_generator)
        self.assertTrue(m.called)

    @patch.object(bilby.gw.detector.Interferometer, 'inject_signal')
    def test_inject_signal_with_inj_pol(self, m):
        self.ifo_list.inject_signal(injection_polarizations=dict(plus=1))
        m.assert_called_with(parameters=None, injection_polarizations=dict(plus=1))
        self.assertEqual(len(self.ifo_list), m.call_count)

    @patch.object(bilby.gw.detector.Interferometer, 'inject_signal')
    def test_inject_signal_returns_expected_polarisations(self, m):
        m.return_value = dict(plus=1, cross=2)
        injection_polarizations = dict(plus=1, cross=2)
        ifos_pol = self.ifo_list.inject_signal(injection_polarizations=injection_polarizations)
        self.assertDictEqual(self.ifo1.inject_signal(injection_polarizations=injection_polarizations), ifos_pol[0])
        self.assertDictEqual(self.ifo2.inject_signal(injection_polarizations=injection_polarizations), ifos_pol[1])

    @patch.object(bilby.gw.detector.Interferometer, 'save_data')
    def test_save_data(self, m):
        self.ifo_list.save_data(outdir='test_outdir', label='test_outdir')
        m.assert_called_with(outdir='test_outdir', label='test_outdir')
        self.assertEqual(len(self.ifo_list), m.call_count)

    def test_number_of_interferometers(self):
        self.assertEqual(len(self.ifo_list), self.ifo_list.number_of_interferometers)

    def test_duration(self):
        self.assertEqual(self.ifo1.strain_data.duration, self.ifo_list.duration)
        self.assertEqual(self.ifo2.strain_data.duration, self.ifo_list.duration)

    def test_sampling_frequency(self):
        self.assertEqual(self.ifo1.strain_data.sampling_frequency, self.ifo_list.sampling_frequency)
        self.assertEqual(self.ifo2.strain_data.sampling_frequency, self.ifo_list.sampling_frequency)

    def test_start_time(self):
        self.assertEqual(self.ifo1.strain_data.start_time, self.ifo_list.start_time)
        self.assertEqual(self.ifo2.strain_data.start_time, self.ifo_list.start_time)

    def test_frequency_array(self):
        self.assertTrue(np.array_equal(self.ifo1.strain_data.frequency_array, self.ifo_list.frequency_array))
        self.assertTrue(np.array_equal(self.ifo2.strain_data.frequency_array, self.ifo_list.frequency_array))

    def test_append_with_ifo(self):
        self.ifo_list.append(self.ifo2)
        names = [ifo.name for ifo in self.ifo_list]
        self.assertListEqual([self.ifo1.name, self.ifo2.name, self.ifo2.name], names)

    def test_append_with_ifo_list(self):
        self.ifo_list.append(self.ifo_list)
        names = [ifo.name for ifo in self.ifo_list]
        self.assertListEqual([self.ifo1.name, self.ifo2.name, self.ifo1.name, self.ifo2.name], names)

    def test_extend(self):
        self.ifo_list.extend(self.ifo_list)
        names = [ifo.name for ifo in self.ifo_list]
        self.assertListEqual([self.ifo1.name, self.ifo2.name, self.ifo1.name, self.ifo2.name], names)

    def test_insert(self):
        new_ifo = self.ifo1
        new_ifo.name = 'name3'
        self.ifo_list.insert(1, new_ifo)
        names = [ifo.name for ifo in self.ifo_list]
        self.assertListEqual([self.ifo1.name, new_ifo.name, self.ifo2.name], names)

    def test_to_and_from_hdf5_loading(self):
        if sys.version_info[0] < 3:
            with self.assertRaises(NotImplementedError):
                self.ifo_list.to_hdf5(outdir='outdir', label='test')
        else:
            self.ifo_list.to_hdf5(outdir='outdir', label='test')
            filename = 'outdir/test_name1name2.h5'
            recovered_ifo = bilby.gw.detector.InterferometerList.from_hdf5(filename)
            self.assertListEqual(self.ifo_list, recovered_ifo)

    def test_to_and_from_hdf5_wrong_class(self):
        if sys.version_info[0] < 3:
            pass
        else:
            dd.io.save('./outdir/psd.h5', self.ifo_list[0].power_spectral_density)
            filename = self.ifo_list._hdf5_filename_from_outdir_label(
                outdir='outdir', label='psd')
            with self.assertRaises(TypeError):
                bilby.gw.detector.InterferometerList.from_hdf5(filename)

    def test_plot_data(self):
        ifos = bilby.gw.detector.InterferometerList(['H1', 'L1'])
        ifos.set_strain_data_from_power_spectral_densities(2048, 4)
        ifos.plot_data(outdir=self.outdir)

        ifos = bilby.gw.detector.InterferometerList(['H1', 'L1', 'V1'])
        ifos.set_strain_data_from_power_spectral_densities(2048, 4)
        ifos.plot_data(outdir=self.outdir)


class TestPowerSpectralDensityWithoutFiles(unittest.TestCase):

    def setUp(self):
        self.frequency_array = np.array([1., 2., 3.])
        self.psd_array = np.array([16., 25., 36.])
        self.asd_array = np.array([4., 5., 6.])

    def tearDown(self):
        del self.frequency_array
        del self.psd_array
        del self.asd_array

    def test_init_with_asd_array(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, asd_array=self.asd_array)
        self.assertTrue(np.array_equal(self.frequency_array, psd.frequency_array))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))

    def test_init_with_psd_array(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, psd_array=self.psd_array)
        self.assertTrue(np.array_equal(self.frequency_array, psd.frequency_array))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))

    def test_setting_asd_array_after_init(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array)
        psd.asd_array = self.asd_array
        self.assertTrue(np.array_equal(self.frequency_array, psd.frequency_array))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))

    def test_setting_psd_array_after_init(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array)
        psd.psd_array = self.psd_array
        self.assertTrue(np.array_equal(self.frequency_array, psd.frequency_array))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))

    def test_power_spectral_density_interpolated_from_asd_array(self):
        expected = np.array([25.])
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, asd_array=self.asd_array)
        self.assertEqual(expected, psd.power_spectral_density_interpolated(2))

    def test_power_spectral_density_interpolated_from_psd_array(self):
        expected = np.array([25.])
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, psd_array=self.psd_array)
        self.assertEqual(expected, psd.power_spectral_density_interpolated(2))

    def test_from_amplitude_spectral_density_array(self):
        actual = bilby.gw.detector.PowerSpectralDensity.from_amplitude_spectral_density_array(
            frequency_array=self.frequency_array, asd_array=self.asd_array)
        self.assertTrue(np.array_equal(self.psd_array, actual.psd_array))
        self.assertTrue(np.array_equal(self.asd_array, actual.asd_array))

    def test_from_power_spectral_density_array(self):
        actual = bilby.gw.detector.PowerSpectralDensity.from_power_spectral_density_array(
            frequency_array=self.frequency_array, psd_array=self.psd_array)
        self.assertTrue(np.array_equal(self.psd_array, actual.psd_array))
        self.assertTrue(np.array_equal(self.asd_array, actual.asd_array))

    def test_repr(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, psd_array=self.psd_array)
        expected = 'PowerSpectralDensity(frequency_array={}, psd_array={}, asd_array={})'.format(self.frequency_array,
                                                                                                 self.psd_array,
                                                                                                 self.asd_array)
        self.assertEqual(expected, repr(psd))


class TestPowerSpectralDensityWithFiles(unittest.TestCase):

    def setUp(self):
        self.dir = os.path.join(os.path.dirname(__file__), 'noise_curves')
        os.mkdir(self.dir)
        self.asd_file = os.path.join(os.path.dirname(__file__), 'noise_curves', 'asd_test_file.txt')
        self.psd_file = os.path.join(os.path.dirname(__file__), 'noise_curves', 'psd_test_file.txt')
        with open(self.asd_file, 'w') as f:
            f.write('1.\t1.0e-21\n2.\t2.0e-21\n3.\t3.0e-21')
        with open(self.psd_file, 'w') as f:
            f.write('1.\t1.0e-42\n2.\t4.0e-42\n3.\t9.0e-42')
        self.frequency_array = np.array([1.0, 2.0, 3.0])
        self.asd_array = np.array([1.0e-21, 2.0e-21, 3.0e-21])
        self.psd_array = np.array([1.0e-42, 4.0e-42, 9.0e-42])

    def tearDown(self):
        os.remove(self.asd_file)
        os.remove(self.psd_file)
        os.rmdir(self.dir)
        del self.dir
        del self.asd_array
        del self.psd_array
        del self.asd_file
        del self.psd_file

    def test_init_with_psd_file(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, psd_file=self.psd_file)
        self.assertEqual(self.psd_file, psd.psd_file)
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))
        self.assertTrue(np.allclose(self.asd_array, psd.asd_array, atol=1e-30))

    def test_init_with_asd_file(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, asd_file=self.asd_file)
        self.assertEqual(self.asd_file, psd.asd_file)
        self.assertTrue(np.allclose(self.psd_array, psd.psd_array, atol=1e-60))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))

    def test_setting_psd_array_after_init(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array)
        psd.psd_file = self.psd_file
        self.assertEqual(self.psd_file, psd.psd_file)
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))
        self.assertTrue(np.allclose(self.asd_array, psd.asd_array, atol=1e-30))

    def test_init_with_asd_array_after_init(self):
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array)
        psd.asd_file = self.asd_file
        self.assertEqual(self.asd_file, psd.asd_file)
        self.assertTrue(np.allclose(self.psd_array, psd.psd_array, atol=1e-60))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))

    def test_power_spectral_density_interpolated_from_asd_file(self):
        expected = np.array([4.0e-42])
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, asd_file=self.asd_file)
        self.assertTrue(np.allclose(expected, psd.power_spectral_density_interpolated(2), atol=1e-60))

    def test_power_spectral_density_interpolated_from_psd_file(self):
        expected = np.array([4.0e-42])
        psd = bilby.gw.detector.PowerSpectralDensity(frequency_array=self.frequency_array, psd_file=self.psd_file)
        self.assertAlmostEqual(expected, psd.power_spectral_density_interpolated(2))

    def test_from_amplitude_spectral_density_file(self):
        psd = bilby.gw.detector.PowerSpectralDensity.from_amplitude_spectral_density_file(asd_file=self.asd_file)
        self.assertEqual(self.asd_file, psd.asd_file)
        self.assertTrue(np.allclose(self.psd_array, psd.psd_array, atol=1e-60))
        self.assertTrue(np.array_equal(self.asd_array, psd.asd_array))

    def test_from_power_spectral_density_file(self):
        psd = bilby.gw.detector.PowerSpectralDensity.from_power_spectral_density_file(psd_file=self.psd_file)
        self.assertEqual(self.psd_file, psd.psd_file)
        self.assertTrue(np.array_equal(self.psd_array, psd.psd_array))
        self.assertTrue(np.allclose(self.asd_array, psd.asd_array, atol=1e-30))

    def test_from_aligo(self):
        psd = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        expected_filename = 'aLIGO_ZERO_DET_high_P_psd.txt'
        expected = bilby.gw.detector.PowerSpectralDensity(psd_file=expected_filename)
        actual_filename = psd.psd_file.split('/')[-1]
        self.assertEqual(expected_filename, actual_filename)
        self.assertTrue(np.allclose(expected.psd_array, psd.psd_array, atol=1e-60))
        self.assertTrue(np.array_equal(expected.asd_array, psd.asd_array))

    def test_check_file_psd_file_set_to_asd_file(self):
        logger = logging.getLogger('bilby')
        m = MagicMock()
        logger.warning = m
        psd = bilby.gw.detector.PowerSpectralDensity(psd_file=self.asd_file)
        self.assertEqual(4, m.call_count)

    def test_check_file_not_called_psd_file_set_to_psd_file(self):
        logger = logging.getLogger('bilby')
        m = MagicMock()
        logger.warning = m
        psd = bilby.gw.detector.PowerSpectralDensity(psd_file=self.psd_file)
        self.assertEqual(0, m.call_count)

    def test_check_file_asd_file_set_to_psd_file(self):
        logger = logging.getLogger('bilby')
        m = MagicMock()
        logger.warning = m
        psd = bilby.gw.detector.PowerSpectralDensity(asd_file=self.psd_file)
        self.assertEqual(4, m.call_count)

    def test_check_file_not_called_asd_file_set_to_asd_file(self):
        logger = logging.getLogger('bilby')
        m = MagicMock()
        logger.warning = m
        psd = bilby.gw.detector.PowerSpectralDensity(asd_file=self.asd_file)
        self.assertEqual(0, m.call_count)

    def test_from_frame_file(self):
        expected_frequency_array = np.array([1., 2., 3.])
        expected_psd_array = np.array([16., 25., 36.])
        with mock.patch('bilby.gw.detector.InterferometerStrainData.set_from_frame_file') as m:
            with mock.patch('bilby.gw.detector.InterferometerStrainData.create_power_spectral_density') as n:
                n.return_value = expected_frequency_array, expected_psd_array
                psd = bilby.gw.detector.PowerSpectralDensity.from_frame_file(frame_file=self.asd_file,
                                                                             psd_start_time=0,
                                                                             psd_duration=4)
                self.assertTrue(np.array_equal(expected_frequency_array, psd.frequency_array))
                self.assertTrue(np.array_equal(expected_psd_array, psd.psd_array))

    def test_repr(self):
        psd = bilby.gw.detector.PowerSpectralDensity(psd_file=self.psd_file)
        expected = 'PowerSpectralDensity(psd_file=\'{}\', asd_file=\'{}\')'.format(self.psd_file, None)
        self.assertEqual(expected, repr(psd))


class TestPowerSpectralDensityEquals(unittest.TestCase):

    def setUp(self):
        self.psd_from_file_1 = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.psd_from_file_2 = bilby.gw.detector.PowerSpectralDensity.from_aligo()
        self.frequency_array = np.linspace(1, 100)
        self.psd_array = np.linspace(1, 100)
        self.psd_from_array_1 = bilby.gw.detector.PowerSpectralDensity. \
            from_power_spectral_density_array(frequency_array=self.frequency_array, psd_array=self.psd_array)
        self.psd_from_array_2 = bilby.gw.detector.PowerSpectralDensity. \
            from_power_spectral_density_array(frequency_array=self.frequency_array, psd_array=self.psd_array)

    def tearDown(self):
        del self.psd_from_file_1
        del self.psd_from_file_2
        del self.frequency_array
        del self.psd_array
        del self.psd_from_array_1
        del self.psd_from_array_2

    def test_eq_true_from_array(self):
        self.assertEqual(self.psd_from_array_1, self.psd_from_array_2)

    def test_eq_true_from_file(self):
        self.assertEqual(self.psd_from_file_1, self.psd_from_file_2)

    def test_eq_false_different_psd_file_name(self):
        self.psd_from_file_1._psd_file = 'some_other_name'
        self.assertNotEqual(self.psd_from_file_1, self.psd_from_file_2)

    def test_eq_false_different_asd_file_name(self):
        self.psd_from_file_1._psd_file = None
        self.psd_from_file_2._psd_file = None
        self.psd_from_file_1._asd_file = 'some_name'
        self.psd_from_file_2._asd_file = 'some_other_name'
        self.assertNotEqual(self.psd_from_file_1, self.psd_from_file_2)

    def test_eq_false_different_frequency_array(self):
        self.psd_from_file_1.frequency_array[0] = 0.5
        self.psd_from_array_1.frequency_array[0] = 0.5
        self.assertNotEqual(self.psd_from_file_1, self.psd_from_file_2)
        self.assertNotEqual(self.psd_from_array_1, self.psd_from_array_2)

    def test_eq_false_different_psd(self):
        self.psd_from_file_1.psd_array[0] = 0.53544321
        self.psd_from_array_1.psd_array[0] = 0.53544321
        self.assertNotEqual(self.psd_from_file_1, self.psd_from_file_2)
        self.assertNotEqual(self.psd_from_array_1, self.psd_from_array_2)

    def test_eq_false_different_asd(self):
        self.psd_from_file_1.asd_array[0] = 0.53544321
        self.psd_from_array_1.asd_array[0] = 0.53544321
        self.assertNotEqual(self.psd_from_file_1, self.psd_from_file_2)
        self.assertNotEqual(self.psd_from_array_1, self.psd_from_array_2)


if __name__ == '__main__':
    unittest.main()
